@@ -6,12 +6,20 @@ Simple CGATS file parser class
 Copyright (C) 2008 Florian Hoech
 """
 
-import os, re, sys
+from __future__ import with_statement
+import math, os, re, sys
 
 import colormath
 
 from safe_print import safe_print
-from util_io import StringIOu as StringIO
+from util_io import GzipFileProper, StringIOu as StringIO
+
+
+def get_device_value_labels(color_rep=None):
+	return filter(bool, map(lambda v: v[1] if not color_rep or v[0] == color_rep
+									  else False,
+							{"CMYK": ("CMYK_C", "CMYK_M", "CMYK_Y", "CMYK_K"),
+							 "RGB": ("RGB_R", "RGB_G", "RGB_B")}.iteritems()))
 
 
 def rpad(value, width):
@@ -48,6 +56,66 @@ def rcut(value, width):
 	return strval
 
 
+def sort_RGB_gray_to_top(a, b):
+	if a[0] == a[1] == a[2]:
+		if b[0] == b[1] == b[2] and a[0] > b[0]:
+			return 1
+		return -1
+	else:
+		return 0
+
+
+def sort_RGB_white_to_top(a, b):
+	sum1, sum2 = sum(a[:3]), sum(b[:3])
+	if sum1 == 300:
+		return -1
+	else:
+		return 0
+
+
+def sort_by_HSV(a, b):
+	a = list(colormath.RGB2HSV(*a[:3]))
+	b = list(colormath.RGB2HSV(*b[:3]))
+	a[0] = round(a[0], 12)
+	b[0] = round(b[0], 12)
+	if a > b:
+		return 1
+	elif a < b:
+		return -1
+	else:
+		return 0
+
+
+def sort_by_RGB(a, b):
+	if a[:3] > b[:3]:
+		return 1
+	elif a[:3] < b[:3]:
+		return -1
+	else:
+		return 0
+
+
+def sort_by_RGB_sum(a, b):
+	sum1, sum2 = sum(a[:3]), sum(b[:3])
+	if sum1 > sum2:
+		return 1
+	elif sum1 < sum2:
+		return -1
+	else:
+		return 0
+
+
+def sort_by_L(a, b):
+	Lab1 = colormath.XYZ2Lab(*a[3:])
+	Lab2 = colormath.XYZ2Lab(*b[3:])
+	if Lab1[0] > Lab2[0]:
+		return 1
+	elif Lab1[0] < Lab2[0]:
+		return -1
+	else:
+		return 0
+
+
 class CGATSError(Exception):
 	pass
 
@@ -85,7 +153,7 @@ class CGATS(dict):
 	datetime = None
 	filename = None
 	key = None
-	modified = False
+	_modified = False
 	mtime = None
 	parent = None
 	root = None
@@ -200,7 +268,7 @@ class CGATS(dict):
 								context = context.add_data({key: value.strip('"')})
 							else:
 								context = context.add_data({key: ''})
-				elif line and values[0] not in ('Comment:', 'Date:') and \
+				elif values and values[0] not in ('Comment:', 'Date:') and \
 				     len(line) >= 3 and not re.search("[^ 0-9A-Za-z]", line):
 					context = self.add_data(line)
 			self.setmodified(False)
@@ -214,11 +282,6 @@ class CGATS(dict):
 		self.setmodified()
 
 	def __getattr__(self, name):
-		if name == 'modified':
-			try:
-				return getattr(self.root, name)
-			except AttributeError:
-				pass
 		if name in self:
 			return self[name]
 		else:
@@ -291,8 +354,8 @@ class CGATS(dict):
 	
 	def setmodified(self, modified=True):
 		""" Set 'modified' state on the 'root' object. """
-		if self.root and self.root.modified != modified:
-			object.__setattr__(self.root, 'modified', modified)
+		if self.root and self.root._modified != modified:
+			object.__setattr__(self.root, '_modified', modified)
 	
 	def __str__(self):
 		result = []
@@ -376,7 +439,10 @@ class CGATS(dict):
 			context['KEYWORDS'].root = self.root
 			context['KEYWORDS'].type = 'KEYWORDS'
 		if not keyword in context['KEYWORDS'].values():
-			context['KEYWORDS'][len(context['KEYWORDS'])] = keyword
+			newkey = len(context['KEYWORDS'])
+			while newkey in context['KEYWORDS']:
+				newkey += 1
+			context['KEYWORDS'][newkey] = keyword
 		if value != None:
 			context[keyword] = value
 	
@@ -396,7 +462,7 @@ class CGATS(dict):
 			context = self.parent.parent
 		else:
 			context = self
-		for key in context['KEYWORDS']:
+		for key in context['KEYWORDS'].keys():
 			if context['KEYWORDS'][key] == keyword:
 				del context['KEYWORDS'][key]
 		if remove_value:
@@ -410,13 +476,108 @@ class CGATS(dict):
 		""" Append data. Also see add_data method. """
 		self.add_data(data)
 	
+	def get_data(self, field_names=None):
+		data = self.queryv1("DATA")
+		if not data:
+			return False
+		elif field_names:
+			data = data.queryi(field_names)
+		return data
+	
+	def get_RGB_XYZ_values(self):
+		field_names = ("RGB_R", "RGB_G", "RGB_B", "XYZ_X", "XYZ_Y", "XYZ_Z")
+		data = self.get_data(field_names)
+		if not data:
+			return False
+		valueslist = []
+		for key, item in data.iteritems():
+			values = []
+			for field_name in field_names:
+				values.append(item[field_name])
+			valueslist.append(values)
+		return data, valueslist
+	
+	def set_RGB_XYZ_values(self, valueslist):
+		field_names = ("RGB_R", "RGB_G", "RGB_B", "XYZ_X", "XYZ_Y", "XYZ_Z")
+		for i, values in enumerate(valueslist):
+			for j, field_name in enumerate(field_names):
+				self[i][field_name] = values[j]
+	
+	def checkerboard(self):
+		data, valueslist = self.get_RGB_XYZ_values()
+		if not valueslist:
+			return False
+		valueslist *= 2
+		valueslist.sort(sort_by_L)
+		valueslist.sort(sort_RGB_white_to_top)
+		split = len(valueslist) / 2
+		valueslist1 = valueslist[:split]
+		valueslist2 = valueslist[split:]
+		valueslist2.reverse()
+		valueslist = valueslist1 + valueslist2
+		checkerboard = []
+		for i in xrange(split):
+			if i % 2 == 1:
+				i = len(valueslist) - 1 - i
+			values = valueslist[i]
+			checkerboard.append(values)
+		return data.set_RGB_XYZ_values(checkerboard)
+	
+	def sort_RGB_gray_to_top(self):
+		data, valueslist = self.get_RGB_XYZ_values()
+		if not valueslist:
+			return False
+		valueslist.sort(sort_RGB_gray_to_top)
+		return data.set_RGB_XYZ_values(valueslist)
+	
+	def sort_RGB_white_to_top(self):
+		data, valueslist = self.get_RGB_XYZ_values()
+		if not valueslist:
+			return False
+		valueslist.sort(sort_RGB_white_to_top)
+		return data.set_RGB_XYZ_values(valueslist)
+	
+	def sort_by_HSV(self):
+		data, valueslist = self.get_RGB_XYZ_values()
+		if not valueslist:
+			return False
+		valueslist.sort(sort_by_HSV)
+		return data.set_RGB_XYZ_values(valueslist)
+	
+	def sort_by_L(self):
+		data, valueslist = self.get_RGB_XYZ_values()
+		if not valueslist:
+			return False
+		valueslist.sort(sort_by_L)
+		return data.set_RGB_XYZ_values(valueslist)
+	
+	def sort_by_RGB(self):
+		data, valueslist = self.get_RGB_XYZ_values()
+		if not valueslist:
+			return False
+		valueslist.sort(sort_by_RGB)
+		return data.set_RGB_XYZ_values(valueslist)
+	
+	def sort_by_RGB_sum(self):
+		data, valueslist = self.get_RGB_XYZ_values()
+		if not valueslist:
+			return False
+		valueslist.sort(sort_by_RGB_sum)
+		return data.set_RGB_XYZ_values(valueslist)
+	
+	@property
+	def modified(self):
+		if self.root:
+			return self.root._modified
+		return self._modified
+	
 	def moveby1(self, start, inc=1):
 		"""
 		Move items from start by icrementing or decrementing their key by inc.
 		"""
-		r = range(start, len(self) + 1)
+		r = xrange(start, len(self) + 1)
 		if inc > 0:
-			r.reverse()
+			r = reversed(r)
 		for key in r:
 			if key in self:
 				if key + inc < 0:
@@ -591,6 +752,239 @@ class CGATS(dict):
 											 self.type)
 		return context
 	
+	def export_vrml(self, filename, devicelocations=True, RGB_black_offset=40,
+					normalize_RGB_white=False, compress=True):
+		data = self.queryv1("DATA")
+		if self.queryv1("ACCURATE_EXPECTED_VALUES") == "true":
+			cat = "Bradford"
+		else:
+			cat = "XYZ scaling"
+		radius = 15.0 / (len(data) ** (1.0 / 3.0))
+		white = data.queryi1({"RGB_R": 100, "RGB_G": 100, "RGB_B": 100})
+		if white:
+			white = colormath.get_whitepoint((white["XYZ_X"],
+											  white["XYZ_Y"],
+											  white["XYZ_Z"]))
+		vrml = """#VRML V2.0 utf8
+
+Transform {
+	children [
+
+		NavigationInfo {
+			type "EXAMINE"
+		}
+
+		DirectionalLight {
+			direction 0 0 -1
+			direction 0 -1 0
+		}
+
+		Viewpoint {
+			position 0 0 340
+		}
+
+		#%(axes)s
+%(children)s
+	]
+}
+"""
+		child = """		# Sphere
+		Transform {
+			translation %(x).6f %(y).6f %(z).6f
+			children [
+				Shape{
+					geometry Sphere { radius %(radius).6f}
+					appearance Appearance { material Material { diffuseColor %(R).6f %(G).6f %(B).6f} }
+				}
+			]
+		}
+"""
+		if devicelocations:
+			axes = ""
+		else:
+			axes = """# L* axis
+		Transform {
+			translation 0.0 0.0 0.0
+			children [
+				Shape {
+					geometry Box { size 2.0 2.0 100.0 }
+					appearance Appearance {
+						material Material { diffuseColor 0.7 0.7 0.7 }
+					}
+				}
+			]
+		}
+		# L* axis label
+		Transform {
+			translation -3.0 -2.0 55.0
+			children [
+				Shape {
+					geometry Text {
+						string ["L"]
+						fontStyle FontStyle { family "SANS" style "BOLD" size 10.0 }
+					}
+					appearance Appearance {
+						material Material { diffuseColor 0.7 0.7 0.7}
+					}
+				}
+			]
+		}
+		# +a axis
+		Transform {
+			translation 50.0 0.0 -50.0
+			children [
+				Shape {
+					geometry Box { size 100.0 2.0 2.0 }
+					appearance Appearance {
+						material Material { diffuseColor 1.0 0.0 0.0 }
+					}
+				}
+			]
+		}
+		# +a axis label
+		Transform {
+			translation 102.0 -2.0 -50.0
+			children [
+				Shape {
+					geometry Text {
+						string ["+a"]
+						fontStyle FontStyle { family "SANS" style "BOLD" size 10.0 }
+					}
+					appearance Appearance {
+						material Material { diffuseColor 1.0 0.0 0.0}
+					}
+				}
+			]
+		}
+		# -a axis
+		Transform {
+			translation -50.0 0.0 -50.0
+			children [
+				Shape {
+					geometry Box { size 100.0 2.0 2.0 }
+					appearance Appearance {
+						material Material { diffuseColor 0.0 1.0 0.0 }
+					}
+				}
+			]
+		}
+		# -a axis label
+		Transform {
+			translation -108.0 -2.0 -50.0
+			children [
+				Shape {
+					geometry Text {
+						string ["-a"]
+						fontStyle FontStyle { family "SANS" style "BOLD" size 10.0 }
+					}
+					appearance Appearance {
+						material Material { diffuseColor 0.0 1.0 0.0}
+					}
+				}
+			]
+		}
+		# +b axis
+		Transform {
+			translation 0.0 50.0 -50.0
+			children [
+				Shape {
+					geometry Box { size 2.0 100.0 2.0 }
+					appearance Appearance {
+						material Material { diffuseColor 1.0 1.0 0.0 }
+					}
+				}
+			]
+		}
+		# +b axis label
+		Transform {
+			translation -3.0 103.0 -50.0
+			children [
+				Shape {
+					geometry Text {
+						string ["+b"]
+						fontStyle FontStyle { family "SANS" style "BOLD" size 10.0 }
+					}
+					appearance Appearance {
+						material Material { diffuseColor 1.0 1.0 0.0}
+					}
+				}
+			]
+		}
+		# -b axis
+		Transform {
+			translation 0.0 -50.0 -50.0
+			children [
+				Shape {
+					geometry Box { size 2.0 100.0 2.0 }
+					appearance Appearance {
+						material Material { diffuseColor 0.0 0.0 1.0 }
+					}
+				}
+			]
+		}
+		# -b axis label
+		Transform {
+			translation -3.0 -107.0 -50.0
+			children [
+				Shape {
+					geometry Text {
+						string ["-b"]
+						fontStyle FontStyle { family "SANS" style "BOLD" size 10.0 }
+					}
+					appearance Appearance {
+						material Material { diffuseColor 0.0 0.0 1.0}
+					}
+				}
+			]
+		}
+"""
+		children = []
+		for entry in data.itervalues():
+			X, Y, Z = colormath.adapt(entry["XYZ_X"],
+									  entry["XYZ_Y"],
+									  entry["XYZ_Z"],
+									  white,
+									  cat=cat)
+			L, a, b = colormath.XYZ2Lab(X, Y, Z)
+			if devicelocations:
+				# Fudge device locations into Lab space
+				x, y, z = (entry["RGB_G"] - 50,
+						   entry["RGB_B"] - 50,
+						   entry["RGB_R"] - 50)
+			else:
+				x, y, z = a, b, L - 50
+			if RGB_black_offset != 40:
+				# Keep reference hue and saturation
+				# Lab to sRGB using reference black offset of 40 like Argyll CMS
+				R, G, B = colormath.Lab2RGB(L * (100.0 - 40.0) / 100.0 + 40.0,
+											a, b, scale=.7,
+											noadapt=not normalize_RGB_white)
+				H_ref, S_ref, V_ref = colormath.RGB2HSV(R, G, B)
+			# Lab to sRGB using actual black offset
+			R, G, B = colormath.Lab2RGB(L * (100.0 - RGB_black_offset) / 100.0 +
+										RGB_black_offset, a, b, scale=.7,
+										noadapt=not normalize_RGB_white)
+			if RGB_black_offset != 40:
+				H, S, V = colormath.RGB2HSV(R, G, B)
+				# Use reference H and S to go back to RGB
+				R, G, B = colormath.HSV2RGB(H_ref, S_ref, V)
+			children.append(child % {"x": x,
+									 "y": y,
+									 "z": z,
+									 "R": R + .05,
+									 "G": G + .05,
+									 "B": B + .05,
+									 "radius": radius})
+		children = "".join(children)
+		vrml = vrml % {"children": children,
+					   "axes": axes}
+		if compress:
+			writer = GzipFileProper
+		else:
+			writer = open
+		with writer(filename, 'w') as vrmlfile:
+			vrmlfile.write(vrml)
+	
 	@property
 	def NUMBER_OF_FIELDS(self):
 		"""Get number of fields"""
@@ -614,6 +1008,7 @@ class CGATS(dict):
 		Return empty CGATS object if no matching items found.
 		
 		"""
+		modified = self.modified
 		
 		if not get_first:
 			result = CGATS()
@@ -662,9 +1057,10 @@ class CGATS(dict):
 						if get_first:
 							if get_value and isinstance(result_n, dict) and \
 							   len(result_n) == 1:
-								return result_n[0]
+								result = result_n[0]
 							else:
-								return result_n
+								result = result_n
+							break
 						elif len(result_n):
 							if get_value and isinstance(result_n, dict) and \
 							   len(result_n) == 1:
@@ -677,13 +1073,16 @@ class CGATS(dict):
 										  get_first)
 					if result_n != None:
 						if get_first:
-							return result_n
+							result = result_n
+							break
 						elif len(result_n):
 							for i in result_n:
 								n = len(result)
 								if result_n[i] not in result.values():
 									result[n] = result_n[i]
 		
+		if isinstance(result, CGATS):
+			result.setmodified(modified)
 		return result
 	
 	def queryi(self, query, query_value=None):
@@ -709,21 +1108,76 @@ class CGATS(dict):
 		else:
 			key = item
 		maxindex = len(self) - 1
-		result = dict.pop(self, key)
+		result = self[key]
 		if type(key) == int and key != maxindex:
 			self.moveby1(key + 1, -1)
-			dict.pop(self, len(self) - 1)
+		dict.pop(self, len(self) - 1)
 		self.setmodified()
 		return result
 	
-	def scale_rgb(self, factor=2.55):
-		""" Scales RGB by multiplying with factor. """
-		data = self.queryi(("RGB_R", "RGB_G", "RGB_B"))
-		for i in data:
-			for label in ("RGB_R", "RGB_G", "RGB_B"):
-				data[i][label] *= factor
+	def fix_device_values_scaling(self, color_rep=None):
+		"""
+		Attempt to fix device value scaling so that max = 100
+		
+		Return number of fixed DATA sections
+
+		"""
+		fixed = 0
+		for labels in get_device_value_labels(color_rep):
+			for dataset in self.query("DATA").itervalues():
+				for item in dataset.queryi(labels).itervalues():
+					for label in labels:
+						if item[label] > 100:
+							dataset.scale_device_values(color_rep=color_rep)
+							fixed += 1
+							break
+		return fixed
 	
-	def apply_bpc(self):
+	def scale_device_values(self, factor=100.0 / 255, color_rep=None):
+		""" Scales device values by multiplying with factor. """
+		for labels in get_device_value_labels(color_rep):
+			for data in self.queryv("DATA").itervalues():
+				for item in data.queryi(labels).itervalues():
+					for label in labels:
+						item[label] *= factor
+	
+	def adapt(self, whitepoint_source=None, whitepoint_destination=None,
+			  cat="Bradford"):
+		"""
+		Perform chromatic adaptation if possible (needs XYZ or LAB)
+		
+		Return number of affected DATA sections.
+		
+		"""
+		n = 0
+		for dataset in self.query("DATA").itervalues():
+			if not dataset.has_cie():
+				continue
+			if not whitepoint_source:
+				whitepoint_source = dataset.get_white_cie("XYZ")
+			if whitepoint_source:
+				n += 1
+				for item in dataset.queryv1("DATA").itervalues():
+					if "XYZ_X" in item:
+						X, Y, Z = item["XYZ_X"], item["XYZ_Y"], item["XYZ_Z"]
+					else:
+						X, Y, Z = colormath.Lab2XYZ(item["LAB_L"],
+													item["LAB_A"],
+													item["LAB_B"],
+													scale=100)
+					X, Y, Z = colormath.adapt(X, Y, Z,
+											  whitepoint_source,
+											  whitepoint_destination,
+											  cat)
+					if "LAB_L" in item:
+						(item["LAB_L"],
+						 item["LAB_A"],
+						 item["LAB_B"]) = colormath.XYZ2Lab(X, Y, Z)
+					if "XYZ_X" in item:
+						item["XYZ_X"], item["XYZ_Y"], item["XYZ_Z"] = X, Y, Z
+		return n
+	
+	def apply_bpc(self, weight=False):
 		"""
 		Apply black point compensation.
 		
@@ -731,45 +1185,133 @@ class CGATS(dict):
 		Needs a CGATS structure with RGB and XYZ data and atleast one black and
 		white patch.
 		
-		Return True if bpc was applied, False otherwise.
+		Return number of affected DATA sections.
 		
 		"""
-		data = self.queryi(("RGB_R", "RGB_G", "RGB_B", "XYZ_X", "XYZ_Y", "XYZ_Z"))
+		n = 0
+		for dataset in self.query("DATA").itervalues():
+			if dataset.type.strip() == "CAL":
+				labels = ("RGB_R", "RGB_G", "RGB_B")
+				data = dataset.queryi(labels)
 
-		# Get blacks
-		blacks = data.queryi({"RGB_R": 0, "RGB_G": 0, "RGB_B": 0})
-		# Get whites
-		whites = data.queryi({"RGB_R": 100, "RGB_G": 100, "RGB_B": 100})
-		if not blacks or not whites:
-			# Can't apply bpc
-			return False
+				# Get black
+				black1 = data.queryi1({"RGB_I": 0})
+				# Get white
+				white1 = data.queryi1({"RGB_I": 1})
+				if not black1 or not white1:
+					# Can't apply bpc
+					continue
 
-		black = [0, 0, 0]
-		for i in blacks:
-			for j, label in enumerate(("XYZ_X", "XYZ_Y", "XYZ_Z")):
-				black[j] += blacks[i][label]
-			if blacks[i]["XYZ_Y"] == 0:
-				break
-		# Average blacks
-		black = [n / (i + 1.0) for n in black]
+				black = []
+				white = []
+				for label in labels:
+					black.append(black1[label])
+					white.append(white1[label])
+			else:
+				labels = ("XYZ_X", "XYZ_Y", "XYZ_Z")
+				data = dataset.queryi(("RGB_R", "RGB_G", "RGB_B") + labels)
 
-		white = [0, 0, 0]
-		for i in whites:
-			for j, label in enumerate(("XYZ_X", "XYZ_Y", "XYZ_Z")):
-				white[j] += whites[i][label]
-			if whites[i]["XYZ_Y"] == 100:
-				break
-		# Average whites
-		white = [n / (i + 1.0) for n in white]
+				# Get blacks
+				blacks = data.queryi({"RGB_R": 0, "RGB_G": 0, "RGB_B": 0})
+				# Get whites
+				whites = data.queryi({"RGB_R": 100, "RGB_G": 100, "RGB_B": 100})
+				if not blacks or not whites:
+					# Can't apply bpc
+					continue
 
-		# Apply black point compensation
-		for i in data:
-			XYZ = data[i].queryv1(("XYZ_X", "XYZ_Y", "XYZ_Z"))
-			XYZ = colormath.apply_bpc(XYZ[0], XYZ[1], XYZ[2], black, (0, 0, 0), white)
-			for j, label in enumerate(("XYZ_X", "XYZ_Y", "XYZ_Z")):
-				data[i][label] = XYZ[j]
+				black = [0, 0, 0]
+				for i in blacks:
+					if blacks[i]["XYZ_Y"] > black[1]:
+						for j, label in enumerate(labels):
+							black[j] = blacks[i][label]
 
-		return True
+				white = [0, 0, 0]
+				for i in whites:
+					if whites[i]["XYZ_Y"] > white[1]:
+						for j, label in enumerate(labels):
+							white[j] = whites[i][label]
+
+			# Apply black point compensation
+			n += 1
+			for i in data:
+				XYZ = data[i].queryv1(labels)
+				XYZ = colormath.apply_bpc(XYZ[0], XYZ[1], XYZ[2], black,
+										  (0, 0, 0), white, weight)
+				for j, label in enumerate(labels):
+					data[i][label] = max(0.0, XYZ[j])
+
+		return n
+	
+	def get_white_cie(self, colorspace=None):
+		"""
+		Get the 'white' from the CIE values (if any).
+		
+		"""
+		data_format = self.has_cie()
+		if data_format:
+			if "RGB_R" in data_format.values():
+				white = {"RGB_R": 100, "RGB_G": 100, "RGB_B": 100}
+			elif "CMYK_C" in data_format.values():
+				white = {"CMYK_C": 0, "CMYK_M": 0, "CMYK_Y": 0, "CMYK_K": 0}
+			else:
+				return
+			if white:
+				white = self.queryi1(white)
+				if not white:
+					white = self.queryv1("APPROX_WHITE_POINT")
+					if white:
+						try:
+							white = [float(v) for v in white.split()]
+						except ValueError:
+							white = None
+						else:
+							if len(white) == 3:
+								white = [v / white[1] * 100 for v in white]
+								white = {"XYZ_X": white[0],
+										 "XYZ_Y": white[1],
+										 "XYZ_Z": white[2]}
+							else:
+								return
+				if white and (("XYZ_X" in white and
+							   "XYZ_Y" in white and
+							   "XYZ_Z" in white) or ("LAB_L" in white and
+													 "LAB_B" in white and
+													 "LAB_B" in white)):
+					if colorspace == "XYZ":
+						if "XYZ_X" in white:
+							return white["XYZ_X"], white["XYZ_Y"], white["XYZ_Z"]
+						else:
+							return colormath.Lab2XYZ(white["LAB_L"],
+													 white["LAB_A"],
+													 white["LAB_B"],
+													 scale=100)
+					elif colorspace == "Lab":
+						if "LAB_L" in white:
+							return white["LAB_L"], white["LAB_A"], white["LAB_B"]
+						else:
+							return colormath.XYZ2Lab(white["XYZ_X"],
+													 white["XYZ_Y"],
+													 white["XYZ_Z"])
+					return white
+	
+	def has_cie(self):
+		"""
+		Check if DATA_FORMAT defines any CIE XYZ or LAB columns.
+		
+		Return the DATA_FORMAT on success or None on failure.
+		
+		"""
+		data_format = self.queryv1("DATA_FORMAT")
+		if data_format:
+			cie = {}
+			for ch in ("L", "A", "B"):
+				cie[ch] = "LAB_%s" % ch in data_format.values()
+			if len(cie.values()) in (0, 3):
+				for ch in ("X", "Y", "Z"):
+					cie[ch] = "XYZ_%s" % ch in data_format.values()
+				if len(filter(lambda v: v is not None,
+							  cie.itervalues())) in (3, 6):
+					return data_format
 	
 	pop = remove
 	
