@@ -1,7 +1,8 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+import logging
 import math
+import sys
 
 
 def get_transfer_function_phi(alpha, gamma):
@@ -117,29 +118,34 @@ def specialpow(a, b):
 
 rgb_spaces = {
 	# http://brucelindbloom.com/WorkingSpaceInfo.html
+	# ACES: https://github.com/ampas/aces-dev/blob/master/docs/ACES_1.0.1.pdf?raw=true
 	# Adobe RGB: http://www.adobe.com/digitalimag/pdfs/AdobeRGB1998.pdf
+	# DCI P3: http://www.hp.com/united-states/campaigns/workstations/pdfs/lp2480zx-dci--p3-emulation.pdf
+	#         http://dcimovies.com/specification/DCI_DCSS_v12_with_errata_2012-1010.pdf
 	#
-	# name              gamma             white  primaries
-	#                                     point   Rx      Ry      RY          Gx      Gy      GY          Bx      By      BY
-	"Adobe RGB (1998)": (2 + 51 / 256.0,  "D65", (0.6400, 0.3300, 0.297361), (0.2100, 0.7100, 0.627355), (0.1500, 0.0600, 0.075285)),
-	"Apple RGB":        (1.8,             "D65", (0.6250, 0.3400, 0.244634), (0.2800, 0.5950, 0.672034), (0.1550, 0.0700, 0.083332)),
-	"Best RGB":         (2.2,             "D50", (0.7347, 0.2653, 0.228457), (0.2150, 0.7750, 0.737352), (0.1300, 0.0350, 0.034191)),
-	"Beta RGB":         (2.2,             "D50", (0.6888, 0.3112, 0.303273), (0.1986, 0.7551, 0.663786), (0.1265, 0.0352, 0.032941)),
-	"Bruce RGB":        (2.2,             "D65", (0.6400, 0.3300, 0.240995), (0.2800, 0.6500, 0.683554), (0.1500, 0.0600, 0.075452)),
-	"CIE RGB":          (2.2,             "E",   (0.7350, 0.2650, 0.176204), (0.2740, 0.7170, 0.812985), (0.1670, 0.0090, 0.010811)),
-	"ColorMatch RGB":   (1.8,             "D50", (0.6300, 0.3400, 0.274884), (0.2950, 0.6050, 0.658132), (0.1500, 0.0750, 0.066985)),
-	"Don RGB 4":        (2.2,             "D50", (0.6960, 0.3000, 0.278350), (0.2150, 0.7650, 0.687970), (0.1300, 0.0350, 0.033680)),
-	"ECI RGB":          (1.8,             "D50", (0.6700, 0.3300, 0.320250), (0.2100, 0.7100, 0.602071), (0.1400, 0.0800, 0.077679)),
-	"ECI RGB v2":       (-3.0,            "D50", (0.6700, 0.3300, 0.320250), (0.2100, 0.7100, 0.602071), (0.1400, 0.0800, 0.077679)),
-	"Ekta Space PS5":   (2.2,             "D50", (0.6950, 0.3050, 0.260629), (0.2600, 0.7000, 0.734946), (0.1100, 0.0050, 0.004425)),
-	"NTSC RGB":         (2.2,             "C",   (0.6700, 0.3300, 0.298839), (0.2100, 0.7100, 0.586811), (0.1400, 0.0800, 0.114350)),
-	"PAL/SECAM RGB":    (2.2,             "D65", (0.6400, 0.3300, 0.222021), (0.2900, 0.6000, 0.706645), (0.1500, 0.0600, 0.071334)),
-	"ProPhoto RGB":     (1.8,             "D50", (0.7347, 0.2653, 0.288040), (0.1596, 0.8404, 0.711874), (0.0366, 0.0001, 0.000086)),
-	"Rec. 709 RGB":     (-709,            "D65", (0.6400, 0.3300, 0.212656), (0.3000, 0.6000, 0.715158), (0.1500, 0.0600, 0.072186)),
-	"SMPTE-C RGB":      (2.2,             "D65", (0.6300, 0.3400, 0.212395), (0.3100, 0.5950, 0.701049), (0.1550, 0.0700, 0.086556)),
-	"SMPTE 240M RGB":   (-240,            "D65", (0.6300, 0.3400, 0.212395), (0.3100, 0.5950, 0.701049), (0.1550, 0.0700, 0.086556)),
-	"sRGB":             (-2.4,            "D65", (0.6400, 0.3300, 0.212656), (0.3000, 0.6000, 0.715158), (0.1500, 0.0600, 0.072186)),
-	"Wide Gamut RGB":   (2.2,             "D50", (0.7350, 0.2650, 0.258187), (0.1150, 0.8260, 0.724938), (0.1570, 0.0180, 0.016875))
+	# name              gamma             white                     primaries
+	#                                     point                     Rx      Ry      RY          Gx      Gy      GY          Bx      By      BY
+	"ACES RGB":         (1.0,             (0.95265, 1.0, 1.00883), (0.7347, 0.2653, 0.343961), (0.0000, 1.0000, 0.728164), (0.0001,-0.0770,-0.072125)),
+	"Adobe RGB (1998)": (2 + 51 / 256.0,  "D65",                   (0.6400, 0.3300, 0.297361), (0.2100, 0.7100, 0.627355), (0.1500, 0.0600, 0.075285)),
+	"Apple RGB":        (1.8,             "D65",                   (0.6250, 0.3400, 0.244634), (0.2800, 0.5950, 0.672034), (0.1550, 0.0700, 0.083332)),
+	"Best RGB":         (2.2,             "D50",                   (0.7347, 0.2653, 0.228457), (0.2150, 0.7750, 0.737352), (0.1300, 0.0350, 0.034191)),
+	"Beta RGB":         (2.2,             "D50",                   (0.6888, 0.3112, 0.303273), (0.1986, 0.7551, 0.663786), (0.1265, 0.0352, 0.032941)),
+	"Bruce RGB":        (2.2,             "D65",                   (0.6400, 0.3300, 0.240995), (0.2800, 0.6500, 0.683554), (0.1500, 0.0600, 0.075452)),
+	"CIE RGB":          (2.2,             "E",                     (0.7350, 0.2650, 0.176204), (0.2740, 0.7170, 0.812985), (0.1670, 0.0090, 0.010811)),
+	"ColorMatch RGB":   (1.8,             "D50",                   (0.6300, 0.3400, 0.274884), (0.2950, 0.6050, 0.658132), (0.1500, 0.0750, 0.066985)),
+	"DCI P3 RGB":       (2.6,             (0.89459, 1.0, 0.95442), (0.6800, 0.3200, 0.209475), (0.2650, 0.6900, 0.721592), (0.1500, 0.0600, 0.068903)),
+	"Don RGB 4":        (2.2,             "D50",                   (0.6960, 0.3000, 0.278350), (0.2150, 0.7650, 0.687970), (0.1300, 0.0350, 0.033680)),
+	"ECI RGB":          (1.8,             "D50",                   (0.6700, 0.3300, 0.320250), (0.2100, 0.7100, 0.602071), (0.1400, 0.0800, 0.077679)),
+	"ECI RGB v2":       (-3.0,            "D50",                   (0.6700, 0.3300, 0.320250), (0.2100, 0.7100, 0.602071), (0.1400, 0.0800, 0.077679)),
+	"Ekta Space PS5":   (2.2,             "D50",                   (0.6950, 0.3050, 0.260629), (0.2600, 0.7000, 0.734946), (0.1100, 0.0050, 0.004425)),
+	"NTSC RGB":         (2.2,             "C",                     (0.6700, 0.3300, 0.298839), (0.2100, 0.7100, 0.586811), (0.1400, 0.0800, 0.114350)),
+	"PAL/SECAM RGB":    (2.2,             "D65",                   (0.6400, 0.3300, 0.222021), (0.2900, 0.6000, 0.706645), (0.1500, 0.0600, 0.071334)),
+	"ProPhoto RGB":     (1.8,             "D50",                   (0.7347, 0.2653, 0.288040), (0.1596, 0.8404, 0.711874), (0.0366, 0.0001, 0.000086)),
+	"Rec. 709 RGB":     (-709,            "D65",                   (0.6400, 0.3300, 0.212656), (0.3000, 0.6000, 0.715158), (0.1500, 0.0600, 0.072186)),
+	"SMPTE-C RGB":      (2.2,             "D65",                   (0.6300, 0.3400, 0.212395), (0.3100, 0.5950, 0.701049), (0.1550, 0.0700, 0.086556)),
+	"SMPTE 240M RGB":   (-240,            "D65",                   (0.6300, 0.3400, 0.212395), (0.3100, 0.5950, 0.701049), (0.1550, 0.0700, 0.086556)),
+	"sRGB":             (-2.4,            "D65",                   (0.6400, 0.3300, 0.212656), (0.3000, 0.6000, 0.715158), (0.1500, 0.0600, 0.072186)),
+	"Wide Gamut RGB":   (2.2,             "D50",                   (0.7350, 0.2650, 0.258187), (0.1150, 0.8260, 0.724938), (0.1570, 0.0180, 0.016875))
 }
 
 
@@ -215,7 +221,7 @@ def adapt(X, Y, Z, whitepoint_source=None, whitepoint_destination=None,
 							  cat) * (X, Y, Z)
 
 
-def apply_bpc(X, Y, Z, bp_in, bp_out, wp_out="D50"):
+def apply_bpc(X, Y, Z, bp_in, bp_out, wp_out="D50", weight=False):
 	"""
 	Apply black point compensation
 	
@@ -223,7 +229,10 @@ def apply_bpc(X, Y, Z, bp_in, bp_out, wp_out="D50"):
 	wp_out = get_whitepoint(wp_out)
 	XYZ = [X, Y, Z]
 	for i, v in enumerate(XYZ):
-		XYZ[i] = ((wp_out[i] - bp_out[i]) * v - wp_out[i] * (bp_in[i] - bp_out[i])) / (wp_out[i] - bp_in[i])
+		if weight and v > 0:
+			XYZ[i] = ((wp_out[i] - bp_out[i]) * v - wp_out[i] * (bp_in[i] * (bp_in[i] / v) - bp_out[i])) / (wp_out[i] - bp_in[i] * (bp_in[i] / v))
+		else:
+			XYZ[i] = ((wp_out[i] - bp_out[i]) * v - wp_out[i] * (bp_in[i] - bp_out[i])) / (wp_out[i] - bp_in[i])
 	return XYZ
 
 
@@ -231,6 +240,34 @@ def avg(*args):
 	return float(sum(args)) / len(args)
 
 
+def interp(x, xp, fp, left=None, right=None):
+	"""
+	One-dimensional linear interpolation similar to numpy.interp
+	
+	Values do NOT have to be monotonically increasing
+	interp(0, [0, 0], [0, 1]) will return 0
+	
+	"""
+	if x in xp:
+		return fp[xp.index(x)]
+	elif x < fp[0]:
+		return xp[0] if left is None else left
+	elif x > xp[-1]:
+		return fp[-1] if right is None else right
+	else:
+		# Interpolate
+		lower = 0
+		higher = len(fp) - 1
+		for i, v in enumerate(xp):
+			if v < x and i > lower:
+				lower = i
+			elif v > x and i < higher:
+				higher = i
+		step = x - xp[lower]
+		steps = (xp[higher] - xp[lower]) / step
+		return fp[lower] + (fp[higher] - fp[lower]) / steps
+
+
 def compute_bpc(bp_in, bp_out):
 	"""
 	Black point compensation. Implemented as a linear scaling in XYZ. 
@@ -306,9 +343,8 @@ def delta(L1, a1, b1, L2, a2, b2, method="1976", p1=None, p2=None, p3=None):
 			C1 = math.sqrt(math.pow(a1, 2) + math.pow(b1, 2))
 			C2 = math.sqrt(math.pow(a2, 2) + math.pow(b2, 2))
 			dC = C1 - C2
-			da = a1 - a2
-			db = b1 - b2
-			dH = math.sqrt(math.pow(da, 2) + math.pow(db, 2) - math.pow(dC, 2))
+			dH2 = math.pow(a1 - a2, 2) + math.pow(b1 - b2, 2) - math.pow(dC, 2)
+			dH = math.sqrt(dH2) if dH2 > 0 else 0
 			SL = 1.0
 			K1 = 0.048 if textiles else 0.045
 			K2 = 0.014 if textiles else 0.015
@@ -332,9 +368,8 @@ def delta(L1, a1, b1, L2, a2, b2, method="1976", p1=None, p2=None, p3=None):
 			C1 = math.sqrt(math.pow(a1, 2) + math.pow(b1, 2))
 			C2 = math.sqrt(math.pow(a2, 2) + math.pow(b2, 2))
 			dC = C1 - C2
-			da = a1 - a2
-			db = b1 - b2
-			dH = math.sqrt(math.pow(da, 2) + math.pow(db, 2) - math.pow(dC, 2))
+			dH2 = math.pow(a1 - a2, 2) + math.pow(b1 - b2, 2) - math.pow(dC, 2)
+			dH = math.sqrt(dH2) if dH2 > 0 else 0
 			SL = 0.511 if L1 < 16 else (0.040975 * L1) / (1 + 0.01765 * L1)
 			SC = (0.0638 * C1) / (1 + 0.0131 * C1) + 0.638
 			F = math.sqrt(math.pow(C1, 4) / (math.pow(C1, 4) + 1900.0))
@@ -392,7 +427,8 @@ def delta(L1, a1, b1, L2, a2, b2, method="1976", p1=None, p2=None, p3=None):
 			C1 = math.sqrt(math.pow(a1, 2) + math.pow(b1, 2))
 			C2 = math.sqrt(math.pow(a2, 2) + math.pow(b2, 2))
 			dC = C1 - C2
-			dH = math.sqrt(math.pow(a1 - a2, 2) + math.pow(b1 - b2, 2) - math.pow(dC, 2))
+			dH2 = math.pow(a1 - a2, 2) + math.pow(b1 - b2, 2) - math.pow(dC, 2)
+			dH = math.sqrt(dH2) if dH2 > 0 else 0
 			dE = math.sqrt(math.pow(dL, 2) + math.pow(a1 - a2, 2) + math.pow(b1 - b2, 2))
 		
 		return {"E": dE,
@@ -489,7 +525,47 @@ def CIEDCCT2XYZ(T, scale=1.0):
 	T = temperature in Kelvin.
 	
 	"""
-	return xyY2XYZ(*CIEDCCT2xyY(T, scale))
+	xyY = CIEDCCT2xyY(T, scale)
+	if xyY:
+		return xyY2XYZ(*xyY)
+
+
+def HSV2RGB(H, S, V, scale=1.0):
+	if S == 0:
+		return (V * scale, ) * 3
+	else:
+		h = H * 6.0
+		if h == 6:
+			h = 0
+		i = int(h)
+		component_1 = V * (1.0 - S)
+		component_2 = V * (1.0 - S * (h - i))
+		component_3 = V * (1.0 - S * (1.0 - (h - i)))
+		if i == 0:
+			R = V
+			G = component_3
+			B = component_1
+		elif i == 1:
+			R = component_2
+			G = V
+			B = component_1
+		elif i == 2:
+			R = component_1
+			G = V
+			B = component_3
+		elif i == 3:
+			R = component_1
+			G = component_2
+			B = V
+		elif i == 4:
+			R = component_3
+			G = component_1
+			B = V
+		else:
+			R = V
+			G = component_1
+			B = component_2
+		return R * scale, G * scale, B * scale
 
 
 def get_DBL_MIN():
@@ -518,10 +594,32 @@ def get_DBL_MIN():
 DBL_MIN = get_DBL_MIN()
 
 
+def LCHab2Lab(L, C, H):
+	a = C * math.cos(H * math.pi / 180.0)
+	b = C * math.sin(H * math.pi / 180.0)
+	return L, a, b
+
+
+def Lab2LCHab(L, a, b):
+	C = math.sqrt(math.pow(a, 2) + math.pow(b, 2))
+	H = 180.0 * math.atan2(b, a) / math.pi
+	if (H < 0.0):
+		H += 360.0
+	return L, C, H
+
+
+def Lab2Luv(L, a, b, whitepoint=None, scale=100):
+	X, Y, Z = Lab2XYZ(L, a, b, whitepoint, scale)
+	return XYZ2Luv(X, Y, Z, whitepoint)
+
+
 def Lab2RGB(L, a, b, rgb_space=None, scale=1.0, round_=False, clamp=True,
-			whitepoint=None):
+			whitepoint=None, noadapt=False, cat="Bradford"):
 	""" Convert from Lab to RGB """
 	X, Y, Z = Lab2XYZ(L, a, b, whitepoint)
+	if not noadapt:
+		X, Y, Z = adapt(X, Y, Z, whitepoint, rgb_space[1] if rgb_space
+											 else "D65", cat)
 	return XYZ2RGB(X, Y, Z, rgb_space, scale, round_, clamp)
 
 
@@ -609,6 +707,42 @@ def Luv2XYZ(L, u, v, whitepoint=None, scale=1.0):
 	return tuple([v * scale for v in X, Y, Z])
 
 
+def RGB2HSV(R, G, B, scale=1.0):
+	RGB_min = min( R, G, B )
+	RGB_max = max( R, G, B )
+	delta = RGB_max - RGB_min
+	V = RGB_max
+	if ( delta == 0 ):
+		# Gray
+		H = 0
+		S = 0
+	else:
+		S = delta / RGB_max
+		delta_R = (((RGB_max - R) / 6.0) + (delta / 2.0) ) / delta
+		delta_G = (((RGB_max - G) / 6.0) + (delta / 2.0) ) / delta
+		delta_B = (((RGB_max - B) / 6.0) + (delta / 2.0) ) / delta
+		if R == RGB_max:
+			H = delta_B - delta_G
+		elif G == RGB_max:
+			H = (1.0 / 3.0) + delta_R - delta_B
+		elif B == RGB_max:
+			H = (2.0 / 3.0) + delta_G - delta_R
+		if H < 0:
+			H += 1.0
+		if H > 1:
+			H -= 1.0
+	return H * scale, S * scale, V * scale
+
+
+def RGB2Lab(R, G, B, rgb_space=None, whitepoint=None, noadapt=False,
+			cat="Bradford"):
+	X, Y, Z = RGB2XYZ(R, G, B, rgb_space, scale=100)
+	if not noadapt:
+		X, Y, Z = adapt(X, Y, Z, rgb_space[1] if rgb_space else "D65",
+						whitepoint, cat)
+	return XYZ2Lab(X, Y, Z, whitepoint=whitepoint)
+
+
 def RGB2XYZ(R, G, B, rgb_space=None, scale=1.0):
 	"""
 	Convert from RGB to XYZ.
@@ -648,14 +782,31 @@ def RGB2XYZ(R, G, B, rgb_space=None, scale=1.0):
 	   with gamma = 2.2.
 	   
 	"""
-	gamma, whitepoint, rxyY, gxyY, bxyY, matrix = get_rgb_space(rgb_space)
+	trc, whitepoint, rxyY, gxyY, bxyY, matrix = get_rgb_space(rgb_space)
 	RGB = [R, G, B]
+	is_trc = isinstance(trc, (list, tuple))
 	for i, v in enumerate(RGB):
-		RGB[i] = specialpow(v, gamma)
+		if is_trc:
+			gamma = trc[i]
+		else:
+			gamma = trc
+		if isinstance(gamma, (list, tuple)):
+			RGB[i] = interp(v, [n / float(len(gamma) - 1) for n in
+							    xrange(len(gamma))], gamma)
+		else:
+			RGB[i] = specialpow(v, gamma)
 	XYZ = matrix * RGB
 	return tuple(v * scale for v in XYZ)
 
 
+def RGBsaturation(R, G, B, saturation, rgb_space=None):
+	""" (De)saturate a RGB color in CIE xy and return the RGB and xyY values """
+	wx, wy, wY = XYZ2xyY(*RGB2XYZ(1, 1, 1, rgb_space=rgb_space))
+	x, y, Y = XYZ2xyY(*RGB2XYZ(R, G, B, rgb_space=rgb_space))
+	x, y, Y = wx + (x - wx) * saturation,  wy + (y - wy) * saturation, Y
+	return xyY2RGB(x, y, Y, rgb_space=rgb_space), (x, y, Y)
+
+
 def rgb_to_xyz_matrix(rx, ry, gx, gy, bx, by, whitepoint=None, scale=1.0):
 	""" Create and return an RGB to XYZ matrix. """
 	whitepoint = get_whitepoint(whitepoint, scale)
@@ -676,7 +827,7 @@ def get_rgb_space(rgb_space=None, scale=1.0):
 		rgb_space = "sRGB"
 	if isinstance(rgb_space, basestring):
 		rgb_space = rgb_spaces[rgb_space]
-	cachehash = tuple(rgb_space), scale
+	cachehash = id(rgb_space), scale
 	cache = get_rgb_space.cache.get(cachehash, None)
 	if cache:
 		return cache
@@ -750,7 +901,9 @@ def planckianCT2XYZ(T, scale=1.0):
 	T = temperature in Kelvin.
 	
 	"""
-	return xyY2XYZ(*planckianCT2xyY(T, scale))
+	xyY = planckianCT2xyY(T, scale)
+	if xyY:
+		return xyY2XYZ(*xyY)
 
 
 def planckianCT2xyY(T, scale=1.0):
@@ -898,7 +1051,8 @@ def XYZ2CCT(X, Y, Z):
 		[0.32931, 0.36038, -40.770],
 		[0.33724, 0.36051, -116.45]
 	]
-	if ((X < 1.0e-20) and (Y < 1.0e-20) and (Z < 1.0e-20)):
+	if ((X < 1.0e-20 and Y < 1.0e-20 and Z < 1.0e-20) or
+		X + 15.0 * Y + 3.0 * Z == 0):
 		return None	# protect against possible divide-by-zero failure
 	us = (4.0 * X) / (X + 15.0 * Y + 3.0 * Z)
 	vs = (6.0 * Y) / (X + 15.0 * Y + 3.0 * Z)
@@ -952,13 +1106,18 @@ def XYZ2Lab(X, Y, Z, whitepoint=None):
 
 def XYZ2Lu_v_(X, Y, Z, whitepoint=None):
 	""" Convert from XYZ to CIE Lu'v' """
+
+	if X + Y + Z == 0:
+		# We can't check for X == Y == Z == 0 because they may actually add up
+		# to 0, thus resulting in ZeroDivisionError later
+		L, u_, v_ = XYZ2Lu_v_(*get_whitepoint(whitepoint))
+		return 0.0, u_, v_
+
+	Xr, Yr, Zr = get_whitepoint(whitepoint, 100)
 	
-	Xn, Yn, Zn = get_whitepoint(whitepoint, 100)
+	yr = Y / Yr
 	
-	if Y / Yn <= math.pow(6.0 / 29.0, 3):
-		L = math.pow(29.0 / 3.0, 3)
-	else:
-		L = 116.0 * math.pow(Y / Yn, 1.0 / 3.0) - 16.0
+	L = 116.0 * cbrt(yr) - 16.0 if yr > LSTAR_E else LSTAR_K * yr
 	
 	u_ = (4.0 * X) / (X + 15.0 * Y + 3.0 * Z)
 	v_ = (9.0 * Y) / (X + 15.0 * Y + 3.0 * Z)
@@ -968,6 +1127,13 @@ def XYZ2Lu_v_(X, Y, Z, whitepoint=None):
 
 def XYZ2Luv(X, Y, Z, whitepoint=None):
 	""" Convert from XYZ to Luv """
+
+	if X + Y + Z == 0:
+		# We can't check for X == Y == Z == 0 because they may actually add up
+		# to 0, thus resulting in ZeroDivisionError later
+		L, u, v = XYZ2Luv(*get_whitepoint(whitepoint))
+		return 0.0, u, v
+
 	Xr, Yr, Zr = get_whitepoint(whitepoint, 100)
 	
 	yr = Y / Yr
@@ -1022,10 +1188,19 @@ def XYZ2RGB(X, Y, Z, rgb_space=None, scale=1.0, round_=False, clamp=True):
 	   gamma = 2.2.
 	
 	"""
-	gamma, whitepoint, rxyY, gxyY, bxyY, matrix = get_rgb_space(rgb_space)
+	trc, whitepoint, rxyY, gxyY, bxyY, matrix = get_rgb_space(rgb_space)
 	RGB = matrix.inverted() * [X, Y, Z]
+	is_trc = isinstance(trc, (list, tuple))
 	for i, v in enumerate(RGB):
-		RGB[i] = specialpow(v, 1.0 / gamma)
+		if is_trc:
+			gamma = trc[i]
+		else:
+			gamma = trc
+		if isinstance(gamma, (list, tuple)):
+			RGB[i] = interp(v, gamma, [n / float(len(gamma) - 1) for n in
+									   xrange(len(gamma))])
+		else:
+			RGB[i] = specialpow(v, 1.0 / gamma)
 		if clamp:
 			RGB[i] = min(1.0, max(0.0, RGB[i]))
 		RGB[i] *= scale
@@ -1077,6 +1252,587 @@ def xy_CCT_delta(x, y, daylight=True, method=2000):
 	return cct, d
 
 
+def dmatrixz(nrl, nrh, ncl, nch):
+	# Adapted from ArgyllCMS numlib/numsup.c
+
+	#nrl  # Row low index
+	#nrh  # Row high index
+	#ncl  # Col low index
+	#nch  # Col high index
+	m = {}
+
+	if nrh < nrl:  # Prevent failure for 0 dimension
+		nrh = nrl
+	if nch < ncl:
+		nch = ncl
+
+	rows = nrh - nrl + 1
+	cols = nch - ncl + 1
+
+	for i in xrange(rows):
+		m[i + nrl] = {}
+		for j in xrange(cols):
+			m[i][j + ncl] = 0
+
+	return m
+
+
+def dvector(nl, nh):
+	# Adapted from ArgyllCMS numlib/numsup.c
+
+	#nl  # Lowest index
+	#nh  # Highest index
+	return {}
+
+
+def gam_fit(gf, v):
+	# Adapted from ArgyllCMS xicc/xicc.c
+	""" gamma + input offset function handed to powell() """
+	gamma = v[0]
+	rv = 0.0
+
+	if gamma < 0.0:
+		rv += 100.0 * -gamma
+		gamma = 1e-4
+
+	tt = math.pow(gf.roo, 1.0 / gamma)
+	b = tt / (1.0 - tt)  # Offset
+	a = math.pow(1.0 - tt, gamma)  # Gain
+
+	tt = a * math.pow(0.5 + b, gamma)
+	tt = tt - gf.thyr
+	rv += tt * tt
+	
+	return rv
+
+
+def linmin(cp, xi, di, ftol, func, fdata):
+	# Adapted from ArgyllCMS numlib/powell.c
+
+	"""
+	Line bracketing and minimisation routine.
+	Return value at minimum.
+
+	"""
+	POWELL_GOLD = 1.618034
+	POWELL_CGOLD = 0.3819660
+	POWELL_MAXIT = 100
+	#cp  # Start point, and returned value
+	#xi[]  # Search vector
+	#di  # Dimensionality
+	#ftol  # Tolerance to stop on
+	#func  # Error function to evaluate
+	#fdata  # Opaque data for func()
+	#ax, xx, bx  # Search vector multipliers
+	#af, xf, bf  # Function values at those points
+	#xt, XT  # Trial point
+	XT = {}
+
+	if di <= 10:
+		xt = XT
+	else:
+		xt = dvector(0, di-1)  # Vector for trial point
+
+	# --------------------------
+	# First bracket the solution
+
+	logging.debug("linmin: Bracketing solution")
+
+	# The line is measured as startpoint + offset * search vector.
+	# (Search isn't symetric, but it seems to depend on cp being
+	# best current solution ?)
+	ax = 0.0
+	for i in xrange(di):
+		xt[i] = cp[i] + ax * xi[i]
+	af = func(fdata, xt)
+
+	# xx being vector offset 0.618
+	xx =  1.0 / POWELL_GOLD
+	for i in xrange(di):
+		xt[i] = cp[i] + xx * xi[i]
+	xf = func(fdata, xt)
+
+	logging.debug("linmin: Initial points a:%f:%f -> b:%f:%f" % (ax, af, xx, xf))
+
+	# Fix it so that we are decreasing from point a -> x
+	if xf > af:
+		tt = ax
+		ax = xx
+		xx = tt
+		tt = af
+		af = xf
+		xf = tt
+
+	logging.debug("linmin: Ordered Initial points a:%f:%f -> b:%f:%f" % (ax, af,
+																		 xx, xf))
+
+	bx = xx + POWELL_GOLD * (xx-ax)  # Guess b beyond a -> x
+	for i in xrange(di):
+		xt[i] = cp[i] + bx * xi[i]
+	bf = func(fdata, xt)
+
+	logging.debug("linmin: Initial bracket a:%f:%f x:%f:%f b:%f:%f" % (ax, af,
+																	   xx, xf,
+																	   bx, bf))
+
+	# While not bracketed
+	while xf > bf:
+
+		logging.debug("linmin: Not bracketed because xf %f > bf %f" % (xf, bf))
+		logging.debug("        ax = %f, xx = %f, bx = %f" % (ax, xx, bx))
+
+		# Compute ux by parabolic interpolation from a, x & b
+		q = (xx - bx) * (xf - af)
+		r = (xx - ax) * (xf - bf)
+		tt = q - r
+		if tt >= 0.0 and tt < 1e-20:  # If +ve too small
+			tt = 1e-20
+		elif tt <= 0.0 and tt > -1e-20:  # If -ve too small
+			tt = -1e-20
+		ux = xx - ((xx - bx) * q - (xx - ax) * r) / (2.0 * tt)
+		ulim = xx + 100.0 * (bx - xx)  # Extrapolation limit
+
+		if (xx - ux) * (ux - bx) > 0.0:  # u is between x and b
+
+			for i in xrange(di):  # Evaluate u
+				xt[i] = cp[i] + ux * xi[i]
+			uf = func(fdata, xt)
+
+
+			if uf < bf:  # Minimum is between x and b
+				ax = xx
+				af = xf
+				xx = ux
+				xf = uf
+				break
+			elif uf > xf:  # Minimum is between a and u
+				bx = ux
+				bf = uf
+				break
+
+			# Parabolic fit didn't work, look further out in direction of b
+			ux = bx + POWELL_GOLD * (bx - xx)
+
+		elif (bx - ux) * (ux - ulim) > 0.0:  # u is between b and limit
+			for i in xrange(di):  # Evaluate u
+				xt[i] = cp[i] + ux * xi[i]
+			uf = func(fdata, xt)
+
+			if uf > bf:  # Minimum is between x and u
+				ax = xx
+				af = xf
+				xx = bx
+				xf = bf
+				bx = ux
+				bf = uf
+				break
+			xx = bx
+			xf = bf  # Continue looking
+			bx = ux
+			bf = uf
+			ux = bx + POWELL_GOLD * (bx - xx)  # Test beyond b
+
+		elif (ux - ulim) * (ulim - bx) >= 0.0:  # u is beyond limit 
+			ux = ulim
+		else:  # u is to left side of x ?
+			ux = bx + POWELL_GOLD * (bx - xx)
+		# Evaluate u, and move into place at b
+		for i in xrange(di):
+			xt[i] = cp[i] + ux * xi[i]
+		uf = func(fdata, xt)
+		ax = xx
+		af = xf
+		xx = bx
+		xf = bf
+		bx = ux
+		bf = uf
+	logging.debug("linmin: Got bracket a:%f:%f x:%f:%f b:%f:%f" % (ax, af,
+																   xx, xf,
+																   bx, bf))
+	# Got bracketed minimum between a -> x -> b
+
+	# ---------------------------------------
+	# Now use brent minimiser bewteen a and b
+	if True:
+		# a and b bracket solution
+		# x is best function value so far
+		# w is second best function value so far
+		# v is previous second best, or third best
+		# u is most recently tested point
+		#wx, vx, ux  # Search vector multipliers
+		#wf
+		vf = 0.0
+		#uf  # Function values at those points
+		de = 0.0  # Distance moved on previous step
+		e = 0.0  # Distance moved on 2nd previous step
+
+		# Make sure a and b are in ascending order
+		if ax > bx:
+			tt = ax
+			ax = bx
+			bx = tt
+			tt = af
+			af = bf
+			bf = tt
+
+		wx = vx = xx  # Initial values of other center points
+		wf = xf = xf
+
+		for iter in xrange(1, POWELL_MAXIT + 1):
+			mx = 0.5 * (ax + bx)  # m is center of bracket values
+			#if ABSTOL:
+				#tol1 = ftol  # Absolute tollerance
+			#else:
+			tol1 = ftol * abs(xx) + 1e-10
+			tol2 = 2.0 * tol1
+
+			logging.debug("linmin: Got bracket a:%f:%f x:%f:%f b:%f:%f" %
+						  (ax, af, xx, xf, bx, bf))
+
+			# See if we're done
+			if abs(xx - mx) <= (tol2 - 0.5 * (bx - ax)):
+				logging.debug("linmin: We're done because %f <= %f" %
+							  (abs(xx - mx), tol2 - 0.5 * (bx - ax)))
+				break
+
+			if abs(e) > tol1:  # Do a trial parabolic fit
+				r = (xx - wx) * (xf-vf)
+				q = (xx - vx) * (xf-wf)
+				p = (xx - vx) * q - (xx-wx) * r
+				q = 2.0 * (q - r)
+				if q > 0.0:
+					p = -p
+				else:
+					q = -q
+				te = e  # Save previous e value
+				e = de  # Previous steps distance moved
+
+				logging.debug("linmin: Trial parabolic fit")
+
+				if (abs(p) >= abs(0.5 * q * te) or p <= q * (ax - xx) or
+					p >= q * (bx - xx)):
+					# Give up on the parabolic fit, and use the golden section search
+					e = ax - xx if xx >= mx else bx - xx  # Override previous distance moved */
+					de = POWELL_CGOLD * e
+					logging.debug("linmin: Moving to golden section search")
+				else:  # Use parabolic fit
+					de = p / q  # Change in xb
+					ux = xx + de  # Trial point according to parabolic fit
+					if (ux - ax) < tol2 or (bx - ux) < tol2:
+						if (mx - xx) > 0.0:  # Don't use parabolic, use tol1
+							de = tol1  # tol1 is +ve
+						else:
+							de = -tol1
+					logging.debug("linmin: Using parabolic fit")
+			else:  # Keep using the golden section search
+				e = ax - xx if xx >= mx else bx - xx  # Override previous distance moved
+				de = POWELL_CGOLD * e
+				logging.debug("linmin: Continuing golden section search")
+
+			if abs(de) >= tol1:  # If de moves as much as tol1 would
+				ux = xx + de  # use it
+				logging.debug("linmin: ux = %f = xx %f + de %f" % (ux, xx, de))
+			else:  # else move by tol1 in direction de
+				if de > 0.0:
+					ux = xx + tol1
+					logging.debug("linmin: ux = %f = xx %f + tol1 %f" %
+								  (ux, xx, tol1))
+				else:
+					ux = xx - tol1
+					logging.debug("linmin: ux = %f = xx %f - tol1 %f" %
+								  (ux, xx, tol1))
+
+			# Evaluate function
+			for i in xrange(di):
+				xt[i] = cp[i] + ux * xi[i]
+			uf = func(fdata, xt)
+
+			if uf <= xf:  # Found new best solution
+				if ux >= xx:
+					ax = xx
+					af = xf  # New lower bracket
+				else:
+					bx = xx
+					bf = xf  # New upper bracket
+				vx = wx
+				vf = wf  # New previous 2nd best solution
+				wx = xx
+				wf = xf  # New 2nd best solution from previous best
+				xx = ux
+				xf = uf  # New best solution from latest
+				logging.debug("linmin: found new best solution")
+			else:  # Found a worse solution
+				if ux < xx:
+					ax = ux
+					af = uf  # New lower bracket
+				else:
+					bx = ux
+					bf = uf  # New upper bracket
+				if uf <= wf or wx == xx:  # New 2nd best solution, or equal best
+					vx = wx
+					vf = wf  # New previous 2nd best solution
+					wx = ux
+					wf = uf  # New 2nd best from latest 
+				elif uf <= vf or vx == xx or vx == wx:  # New 3rd best, or equal 1st & 2nd
+					vx = ux
+					vf = uf  # New previous 2nd best from latest
+				logging.debug("linmin: found new worse solution")
+		# !!! should do something if iter > POWELL_MAXIT !!!!
+		# Solution is at xx, xf
+
+		# Compute solution vector
+		for i in xrange(di):
+			cp[i] += xx * xi[i]
+
+	return xf
+
+def powell(di, cp, s, ftol, maxit, func, fdata, prog=None, pdata=None):
+	# Adapted from ArgyllCMS powell.c
+
+	"""
+	Standard interface for powell function
+	return True on sucess, False on failure due to excessive iterions
+	Result will be in cp
+	
+	"""
+	DBL_EPSILON = 2.2204460492503131e-016
+	#di  # Dimentionality
+	#cp  # Initial starting point
+	#s  # Size of initial search area
+	#ftol  # Tolerance of error change to stop on
+	#maxit  # Maximum iterations allowed
+	#func  # Error function to evaluate
+	#fdata  # Opaque data needed by function
+	#prog  # Optional progress percentage callback
+	#pdata  # Opaque data needed by prog()
+	
+	#dmtx  # Direction vector
+	#sp  # Sarting point before exploring all the directions
+	#xpt  # Extrapolated point
+	#svec  # Search vector
+	#retv  # Returned function value at p
+	#stopth  # Current stop threshold */
+	startdel = -1.0  # Initial change in function value
+	#curdel  # Current change in function value
+	pc = 0  # Percentage complete
+
+	dmtx = dmatrixz(0, di - 1, 0, di - 1)  # Zero filled
+	spt  = dvector(0, di - 1)
+	xpt  = dvector(0, di - 1)
+	svec = dvector(0, di - 1)
+
+	# Create initial direction matrix by
+	# placing search start on diagonal
+	for i in xrange(di):
+		dmtx[i][i] = s[i]
+		# Save the starting point
+		spt[i] = cp[i]
+
+	if prog:  # Report initial progress
+		prog(pdata, pc)
+
+	# Initial function evaluation
+	retv = func(fdata, cp)
+
+	# Iterate untill we converge on a solution, or give up.
+	for iter in xrange(1, maxit):
+		#lretv  # Last function return value
+		ibig = 0  # Index of biggest delta
+		del_ = 0.0  # Biggest function value decrease
+		#pretv  # Previous function return value
+
+		pretv = retv  # Save return value at top of iteration
+
+		# Loop over all directions in the set
+		for i in xrange(di):
+
+			logging.debug("Looping over direction %d" % i)
+
+			for j in xrange(di):  # Extract this direction to make search vector
+				svec[j] = dmtx[j][i]
+
+			# Minimize in that direction
+			lretv = retv
+			retv = linmin(cp, svec, di, ftol, func, fdata)
+
+			# Record bigest function decrease, and dimension it occured on
+			if abs(lretv - retv) > del_:
+				del_ = abs(lretv - retv)
+				ibig = i
+
+		#if ABSTOL:
+			#stopth = ftol  # Absolute tollerance
+		#else
+		stopth = ftol * 0.5 * (abs(pretv) + abs(retv) + DBL_EPSILON)
+		curdel = abs(pretv - retv)
+		if startdel < 0.0:
+			startdel = curdel
+		elif curdel > 0 and startdel > 0:
+			tt = (100.0 * math.pow((math.log(curdel) - math.log(startdel)) /
+								   (math.log(stopth) - math.log(startdel)),
+								   4.0) + 0.5)
+			if tt > pc and tt < 100:
+				pc = tt
+				if prog:  # Report initial progress
+					prog(pdata, pc)
+
+		# If we have had at least one change of direction and
+		# reached a suitable tollerance, then finish
+		if iter > 1 and curdel <= stopth:
+			logging.debug("Reached stop tollerance because curdel %f <= stopth "
+						  "%f" % (curdel, stopth))
+			break
+		logging.debug("Not stopping because curdel %f > stopth %f" % (curdel,
+																	  stopth))
+
+		for i in xrange(di):
+			svec[i] = cp[i] - spt[i]  # Average direction moved after minimization round
+			xpt[i]  = cp[i] + svec[i]  # Extrapolated point after round of minimization
+			spt[i]  = cp[i]  # New start point for next round
+
+		# Function value at extrapolated point
+		lretv = func(fdata, xpt)
+
+		if lretv < pretv:  # If extrapolation is an improvement
+
+			t1 = pretv - retv - del_
+			t2 = pretv - lretv
+			t = 2.0 * (pretv -2.0 * retv + lretv) * t1 * t1 - del_ * t2 * t2
+			if t < 0.0:
+				# Move to the minimum of the new direction
+				retv = linmin(cp, svec, di, ftol, func, fdata)
+
+				for i in xrange(di):  # Save the new direction
+					dmtx[i][ibig] = svec[i]  # by replacing best previous
+
+	if prog:  # Report final progress
+		prog(pdata, 100)
+
+	if iter < maxit:
+		return True
+
+	logging.debug("powell: returning False due to excessive iterations")
+	return False  # Failed due to execessive iterations
+
+
+def xicc_tech_gamma(egamma, off):
+	# Adapted from ArgyllCMS xicc.c
+
+	"""
+	Given the effective gamma and the output offset Y,
+	return the technical gamma needed for the correct 50% response.
+	
+	"""
+	gf = gam_fits()
+	op = {}
+	sa = {}
+
+	if off <= 0.0:
+		return egamma
+
+	gf.thyr = math.pow(0.5, egamma)  # Advetised 50% target
+	gf.roo = off
+
+	op[0] = egamma
+	sa[0] = 0.1
+
+	if not powell(1, op, sa, 1e-6, 500, gam_fit, gf):
+		logging.warn("Computing effective gamma and input offset is inaccurate")
+
+	return op[0]
+
+
+class gam_fits(object):
+	# Adapted from ArgyllCMS xicc/xicc.c
+
+	def __init__(self, thyr=.2, roo=0):
+		self.thyr = thyr  # 50% input target
+		self.roo = roo  # 0% input target
+
+
+class BT1886(object):
+	# Adapted from ArgyllCMS xicc/xicc.c
+
+	""" BT.1886 like transfer function """
+
+	def __init__(self, matrix, XYZbp, gamma):
+		""" Setup the bt1886_info for the given target """
+		self.bwd_matrix = matrix.inverted()
+		self.fwd_matrix = matrix
+		self.gamma = gamma
+
+		Lab = XYZ2Lab(*XYZbp)
+
+		self.outL = Lab[0]  # For bp blend
+		self.tab = Lab[1:]  # a* b* correction needed
+
+		bkipow = math.pow(XYZbp[1], 1.0 / self.gamma)
+		self.ingo = bkipow / (1.0 - bkipow)  # non-linear Y that makes out black point
+		self.outsc = pow(1.0 - bkipow, self.gamma)  # Scale to restore 1 -> 1
+
+	def apply(self, X, Y, Z):
+		"""
+		Apply BT.1886 black offset and gamma curve to the XYZ out of the input profile.
+		Do this in the colorspace defined by the input profile matrix lookup,
+		so it will be relative XYZ. We assume that BT.1886 does a Rec709 to gamma
+		viewing adjustment, irrespective of the source profile transfer curve.
+		
+		"""
+
+		logging.debug("bt1886 XYZ in %f %f %f" % (X, Y, Z))
+
+		out = self.bwd_matrix * (X, Y, Z)
+
+		logging.debug("bt1886 RGB in %f %f %f" % (out[0], out[1], out[2]))
+
+		for j in xrange(3):
+			vv = out[j]
+		
+			# Convert linear light to Rec709 transfer curve
+			if vv < 0.018:
+				vv = 4.5 * vv
+			else:
+				vv = 1.099 * math.pow(vv, 0.45) - 0.099
+			
+			# Apply input offset & re-scale, and then gamma of 2.4/custom gamma
+			vv = vv + self.ingo
+			
+			if vv > 0.0:
+				vv = self.outsc * math.pow(vv, self.gamma)
+
+			out[j] = vv
+
+		out = self.fwd_matrix * out
+
+		logging.debug("bt1886 RGB bt.1886 %f %f %f" % (out[0], out[1], out[2]))
+
+		out = list(XYZ2Lab(*[v * 100 for v in out]))
+
+		logging.debug("bt1886 Lab after Y adj. %f %f %f" % (out[0], out[1],
+															  out[2]))
+
+		# Blend ab to required black point offset self.tab[] as L approaches black.
+		vv = (out[0] - self.outL) / (100.0 - self.outL)  # 0 at bp, 1 at wp
+		vv = 1.0 - vv
+
+		if vv < 0.0:
+			vv = 0.0
+		elif vv > 1.0:
+			vv = 1.0
+		vv = math.pow(vv, 40.0)
+		out[1] += vv * self.tab[0]
+		out[2] += vv * self.tab[1]
+
+		logging.debug("bt1886 Lab after wp adj. %f %f %f" % (out[0], out[1],
+															   out[2]))
+
+		out = Lab2XYZ(*out)
+
+		logging.debug("bt1886 XYZ out %f %f %f" % (out[0], out[1], out[2]))
+
+		return out
+
+
 class Matrix3x3(list):
 	
 	""" Simple 3x3 matrix """
