@@ -16,7 +16,7 @@ from time import gmtime, strftime, timezone
 if sys.platform == "win32":
 	import _winreg
 
-from argyll_names import viewconds, intents
+from argyll_names import viewconds, intents, video_encodings
 from defaultpaths import appdata, commonappdata
 if sys.platform == "win32":
 	from defaultpaths import commonprogramfiles
@@ -108,6 +108,8 @@ else:
 		if not datahome_default in data_dirs:
 			data_dirs += [datahome_default]
 		data_dirs += [os.path.join(dir_, appname) for dir_ in xdg_data_dirs]
+		data_dirs += [os.path.join(dir_, "color", "argyll") for dir_ in
+					  xdg_data_dirs]
 		del dir_
 	exe_ext = ""
 	profile_ext = ".icc"
@@ -120,11 +122,10 @@ resfiles = [
 	"lang/en.json",
 	"beep.wav",
 	"camera_shutter.wav",
+	"linear.cal",
 	"test.cal",
 	"ref/ClayRGB1998.gam",
-	"ref/Rec709.icm",
 	"ref/sRGB.gam",
-	"ref/sRGB.icm",
 	"ref/verify.ti1",
 	"xrc/3dlut.xrc",
 	"xrc/extra.xrc",
@@ -216,13 +217,16 @@ def get_argyll_data_dir():
 							argyll_data_dirname)
 
 
-def get_display_name(n=None):
+def get_display_name(n=None, include_geometry=False):
 	""" Return name of currently configured display """
 	if n is None:
 		n = getcfg("display.number") - 1
 	displays = getcfg("displays").split(os.pathsep)
 	if n >= 0 and n < len(displays):
-		return displays[n].split("@")[0].strip()
+		if include_geometry:
+			return displays[n]
+		else:
+			return displays[n].split("@")[0].strip()
 	return ""
 
 
@@ -234,7 +238,7 @@ def get_display_number(display_no):
 	except IndexError:
 		pass
 	else:
-		if display == "Web @ localhost":
+		if display in ("Web @ localhost", "Untethered", "madVR"):
 			return 0
 		for i in xrange(wx.Display.GetCount()):
 			geometry = "%i, %i, %ix%i" % tuple(wx.Display(i).Geometry)
@@ -314,6 +318,12 @@ def get_data_path(relpath, rex=None):
 							paths += [os.path.join(curpath, filename)]
 			else:
 				return curpath
+	if paths:
+		paths = [os.pathsep.join(reversed(os.path.split(path))) for path in
+				 paths]
+		paths.sort()
+		paths = [os.path.join(*reversed(path.split(os.pathsep))) for path in
+				 paths]
 	return None if len(paths) == 0 else paths
 
 
@@ -373,8 +383,6 @@ def runtimeconfig(pyfile):
 						  for dir_ in xdg_data_dirs + [xdg_data_home]])
 	if debug:
 		safe_print("[D] Data files search paths:\n[D]", "\n[D] ".join(data_dirs))
-	defaults["3dlut.input.profile"] = get_data_path(os.path.join("ref",
-																 "Rec709.icm")) or ""
 	defaultmmode = defaults["measurement_mode"]
 	defaultptype = defaults["profile.type"]
 	defaultchart = testchart_defaults.get(defaultptype, 
@@ -382,9 +390,8 @@ def runtimeconfig(pyfile):
 	defaults["testchart.file"] = get_data_path(os.path.join("ti1", 
 															defaultchart)) or ""
 	defaults["testchart.file.backup"] = defaults["testchart.file"]
-	defaults["profile_verification_chart"] = get_data_path(os.path.join("ref", 
+	defaults["measurement_report.chart"] = get_data_path(os.path.join("ref", 
 															"verify.ti1")) or ""
-	defaults["gamap_profile"] = get_data_path(os.path.join("ref", "sRGB.icm")) or ""
 	return runtype
 
 # User settings
@@ -393,7 +400,7 @@ cfg = ConfigParser.RawConfigParser()
 cfg.optionxform = str
 
 valid_ranges = {
-	"3dlut.black_point_compensation": [0, 1],
+	"3dlut.bt1886_gamma": [0.000001, 10],
 	"gamma": [0.000001, 10],
 	"trc": [0.000001, 10],
 	"calibration.ambient_viewcond_adjust.lux": [0, sys.maxint],
@@ -402,19 +409,23 @@ valid_ranges = {
 	"calibration.black_point_correction": [0, 1],
 	"calibration.black_point_rate": [0.05, 20],
 	"calibration.luminance": [0.000001, 100000],
-	"gamap_default_intent": [0, 3],
-	"measure.uniformity.show_detail": [0, 1],
-	"profile.black_point_compensation": [0, 1],
+	"measurement_report.bt1886_gamma": [0.01, 10],
+	"tc_export_repeat_patch_max": [1, 1000],
+	"tc_export_repeat_patch_min": [1, 1000],
+	"tc_vrml_black_offset": [0, 40],
 	"webserver.portnumber": [1, 65535],
 	"whitepoint.colortemp": [1000, 15000],
 }
 
 valid_values = {
-	"3dlut.bitdepth.input": [0, 1, 2, 3, 4],
-	"3dlut.bitdepth.output": [0, 1, 2, 3, 4],
-	"3dlut.format": [0, 1, 2],
-	"3dlut.rendering_intent": [0, 1, 2, 3],
-	"3dlut.size": [0, 1, 2],
+	"3dlut.bitdepth.input": [8, 10, 12, 14, 16],
+	"3dlut.bitdepth.output": [8, 10, 12, 14, 16],
+	"3dlut.bt1886_gamma_type": ["b", "B"],
+	"3dlut.encoding.input": video_encodings,
+	"3dlut.encoding.output": video_encodings,
+	"3dlut.format": ["3dl", "cube", "eeColor", "madVR", "mga", "spi3d"],
+	"3dlut.rendering_intent": intents,
+	"3dlut.size": [17, 24, 32, 33, 64, 65],
 	"calibration.quality": ["v", "l", "m", "h", "u"],
 	# Measurement modes as supported by Argyll -y parameter
 	# 'l' = 'n' (non-refresh-type display, e.g. LCD)
@@ -426,31 +437,44 @@ valid_values = {
 	"measurement_mode": [None, "B", "F", "L", "R", "a", "b", "c", "e", "f", "g",
 						 "l", "m", "n", "o", "p", "r", "x", "1", "2", "3", "4", 
 						 "5", "6", "7", "8", "9"],
+	"gamap_default_intent": ["a", "r", "p", "s"],
 	"gamap_perceptual_intent": intents,
 	"gamap_saturation_intent": intents,
 	"gamap_src_viewcond": viewconds,
 	"gamap_out_viewcond": ["mt", "mb", "md", "jm", "jd"],
+	"measure.uniformity.show_detail": [0, 1],
+	"measurement_report.bt1886_gamma_type": ["b", "B"],
+	"profile.black_point_compensation": [0, 1],
 	"profile.install_scope": ["l", "u"],
 	"profile.quality": ["l", "m", "h", "u"],
+	"profile.quality.b2a": ["l", "m", "h", "u", "n", None],
 	"profile.type": ["g", "G", "l", "s", "S", "x", "X"],
 	"tc_algo": ["", "t", "r", "R", "q", "Q", "i", "I"],  # Q = Argyll >= 1.1.0
+	"tc_vrml_use_D50": [0, 1],
 	"trc": ["240", "709", "l", "s"],
 	"trc.type": ["g", "G"],
 	"whitepoint.colortemp.locus": ["t", "T"]
 }
 
 defaults = {
-	"3dlut.black_point_compensation": 0,
-	"3dlut.bitdepth.input": 2,
-	"3dlut.bitdepth.output": 2,
-	"3dlut.format": 0,
+	"3dlut.apply_bt1886_gamma_mapping": 0,
+	"3dlut.bitdepth.input": 10,
+	"3dlut.bitdepth.output": 10,
+	"3dlut.bt1886_gamma": 2.2,
+	"3dlut.bt1886_gamma_type": "b",
+	"3dlut.encoding.input": "n",
+	"3dlut.encoding.input.backup": "n",
+	"3dlut.encoding.output": "n",
+	"3dlut.encoding.output.backup": "n",
+	"3dlut.format": "3dl",
 	"3dlut.input.profile": "",
 	"3dlut.abstract.profile": "",
+	"3dlut.madVR.enable": 1,
 	"3dlut.output.profile": "",
 	"3dlut.output.profile.apply_cal": 1,
-	"3dlut.rendering_intent": 1,
+	"3dlut.rendering_intent": "r",
 	"3dlut.use_abstract_profile": 0,
-	"3dlut.size": 0,
+	"3dlut.size": 17,
 	"allow_skip_sensor_cal": 0,
 	"argyll.debug": 0,
 	"argyll.dir": expanduseru("~"), # directory
@@ -461,6 +485,7 @@ defaults = {
 	"calibration.ambient_viewcond_adjust.lux": 500.0,
 	"calibration.black_luminance": 0.000001,
 	"calibration.black_output_offset": 1.0,
+	"calibration.black_output_offset.backup": 1.0,
 	"calibration.black_point_correction": 0.0,
 	"calibration.black_point_correction.auto": 0,
 	"calibration.black_point_correction_choice.show": 1,
@@ -472,6 +497,8 @@ defaults = {
 	"calibration.luminance": 120.0,
 	"calibration.quality": "m",
 	"calibration.update": 0,
+	"calibration.use_video_lut": 1,
+	"calibration.use_video_lut.backup": 1,
 	"colorimeter_correction.testchart": "ccxx.ti1",
 	"colorimeter_correction_matrix_file": "AUTO:",
 	"comport.number": 1,
@@ -484,17 +511,20 @@ defaults = {
 	"display_lut.link": 1,
 	"display_lut.number": 1,
 	"displays": "",
+	"dry_run": 0,
 	"enumerate_ports.auto": 0,
+	"extra_args.collink": "",
 	"extra_args.colprof": "",
 	"extra_args.dispcal": "",
 	"extra_args.dispread": "",
 	"extra_args.spotread": "",
+	"extra_args.targen": "",
 	"gamap_profile": "",
 	"gamap_perceptual": 0,
 	"gamap_perceptual_intent": "la",
 	"gamap_saturation": 0,
 	"gamap_saturation_intent": "s",
-	"gamap_default_intent": 0,
+	"gamap_default_intent": "p",
 	"gamma": 2.2,
 	"instruments": "",
 	"log.autoshow": int(not(hasattr(sys.stdout, "isatty") and 
@@ -505,10 +535,24 @@ defaults = {
 	"lut_viewer.show": 0,
 	"lut_viewer.show_actual_lut": 0,
 	"measurement_mode": "l",
+	"measurement_mode_unlocked": 1,
 	"measurement_mode.adaptive": 1,
 	"measurement_mode.backup": "l",
 	"measurement_mode.highres": 0,
 	"measurement_mode.projector": 0,
+	"measurement_report.apply_bt1886_gamma_mapping": 0,
+	"measurement_report.bt1886_gamma": 2.2,
+	"measurement_report.bt1886_gamma_type": "b",
+	"measurement_report.chart": "",
+	"measurement_report.chart.fields": "RGB",
+	"measurement_report.devlink_profile": "",
+	"measurement_report.output_profile": "",
+	"measurement_report.whitepoint.simulate": 0,
+	"measurement_report.whitepoint.simulate.relative": 0,
+	"measurement_report.simulation_profile": "",
+	"measurement_report.use_devlink_profile": 0,
+	"measurement_report.use_simulation_profile": 0,
+	"measurement_report.use_simulation_profile_as_output": 0,
 	"measurement.continuous": 0,
 	"measurement.name.expanded": u"",
 	"measurement.play_sound": 1,
@@ -528,6 +572,10 @@ defaults = {
 	"position.profile_info.y": 50,
 	"position.progress.x": 50,
 	"position.progress.y": 50,
+	"position.reportframe.x": 50,
+	"position.reportframe.y": 50,
+	"position.synthiccframe.x": 50,
+	"position.synthiccframe.y": 50,
 	"position.tcgen.x": 50,
 	"position.tcgen.y": 50,
 	"profile.black_point_compensation": 1,
@@ -552,6 +600,7 @@ defaults = {
 	]),
 	"profile.name.expanded": u"",
 	"profile.quality": "h",
+	"profile.quality.b2a": None,
 	"profile.save_path": storage, # directory
 	"profile.type": "s",
 	"profile.update": 0,
@@ -570,31 +619,60 @@ defaults = {
 	"size.profile_info.h": 524,
 	"size.lut_viewer.w": 512,
 	"size.lut_viewer.h": 580,
+	"size.reportframe.w": 512,
+	"size.reportframe.h": 256,
+	"size.synthiccframe.w": 512,
+	"size.synthiccframe.h": 384,
 	"skip_legacy_serial_ports": 1,
 	"sudo.preserve_environment": 1,
 	"tc_adaption": 0.1,
+	"tc_add_ti3_relative": 1,
 	"tc_algo": "",
 	"tc_angle": 0.3333,
+	"tc_black_patches": 4,
+	"tc_export_repeat_patch_max": 1,
+	"tc_export_repeat_patch_min": 1,
 	"tc_filter": 0,
 	"tc_filter_L": 50,
 	"tc_filter_a": 0,
 	"tc_filter_b": 0,
 	"tc_filter_rad": 255,
 	"tc_fullspread_patches": 0,
+	"tc_gamma": 1.0,
 	"tc_gray_patches": 9,
+	"tc_multi_bcc": 0,
+	"tc_multi_bcc_steps": 0,
 	"tc_multi_steps": 3,
+	"tc_neutral_axis_emphasis": 0.5,
+	"tc_dark_emphasis": 0.0,
 	"tc_precond": 0,
 	"tc_precond_profile": "",
+	"tc.saturation_sweeps": 5,
+	"tc.saturation_sweeps.custom.R": 0.0,
+	"tc.saturation_sweeps.custom.G": 0.0,
+	"tc.saturation_sweeps.custom.B": 0.0,
 	"tc_single_channel_patches": 0,
+	"tc_vrml_black_offset": 40,
 	"tc_vrml_lab": 0,
 	"tc_vrml_device": 1,
+	"tc_vrml_use_D50": 0,
 	"tc_white_patches": 4,
 	"tc.show": 0,
+	"testchart.reference": "",
+	"ti3.check_sanity.auto": 0,
 	"trc": 2.2,
+	"trc.backup": 2.2,
 	"trc.should_use_viewcond_adjust.show_msg": 1,
 	"trc.type": "g",
+	"trc.type.backup": "g",
+	"untethered.measure.auto": 1,
+	"untethered.measure.manual.delay": 0.75,
+	"untethered.max_delta.chroma": 0.5,
+	"untethered.min_delta": 1.5,
+	"untethered.min_delta.lightness": 1.0,
 	"update_check": 1,
 	"use_separate_lut_access": 0,
+	"vrml.compress": 1,
 	"webserver.portnumber": 8080,
 	"whitepoint.colortemp": 5000,
 	"whitepoint.colortemp.locus": "t",
@@ -606,8 +684,8 @@ if lcode:
 	defaults["lang"] = lcode.split("_")[0].lower()
 
 testchart_defaults = {
-	"s": {None: "d3-e4-s0-g9-m3-f0-crossover.ti1"},  # shaper + matrix
-	"l": {None: "d3-e4-s0-g49-m0-f0-cubic4-crossover.ti1"},  # lut
+	"s": {None: "d3-e4-s0-g9-m3-b3-f0-crossover.ti1"},  # shaper + matrix
+	"l": {None: "d3-e4-s17-g49-m5-b5-f0.ti1"},  # lut
 	"g": {None: "d3-e4-s3-g3-m0-f0.ti1"}  # gamma + matrix
 }
 
@@ -765,7 +843,7 @@ def get_current_profile(include_display_profile=False):
 def get_display_profile(display_no=None):
 	if display_no is None:
 		display_no = max(getcfg("display.number") - 1, 0)
-	if get_display_name(display_no) == "Web":
+	if get_display_name(display_no) in ("Web", "Untethered", "madVR"):
 		return None
 	import ICCProfile as ICCP
 	try:
@@ -776,11 +854,14 @@ def get_display_profile(display_no=None):
 					exception, fn=log)
 
 
-def get_total_patches(white_patches=None, single_channel_patches=None, 
-					  gray_patches=None, multi_steps=None, 
+def get_total_patches(white_patches=None, black_patches=None,
+					  single_channel_patches=None, gray_patches=None,
+					  multi_steps=None, multi_bcc_steps=None,
 					  fullspread_patches=None):
 	if white_patches is None:
 		white_patches = getcfg("tc_white_patches")
+	if black_patches is None and getcfg("argyll.version") >= "1.6":
+		black_patches = getcfg("tc_black_patches")
 	if single_channel_patches is None:
 		single_channel_patches = getcfg("tc_single_channel_patches")
 	single_channel_patches_total = single_channel_patches * 3
@@ -790,18 +871,30 @@ def get_total_patches(white_patches=None, single_channel_patches=None,
 		gray_patches = 2
 	if multi_steps is None:
 		multi_steps = getcfg("tc_multi_steps")
+	if multi_bcc_steps is None and getcfg("argyll.version") >= "1.6":
+		multi_bcc_steps = getcfg("tc_multi_bcc_steps")
 	if fullspread_patches is None:
 		fullspread_patches = getcfg("tc_fullspread_patches")
 	total_patches = 0
 	if multi_steps > 1:
 		multi_patches = int(math.pow(multi_steps, 3))
+		if multi_bcc_steps > 1:
+			multi_patches += int(math.pow(multi_bcc_steps - 1, 3))
 		total_patches += multi_patches
 		white_patches -= 1 # white always in multi channel patches
 
 		multi_step = 255.0 / (multi_steps - 1)
 		multi_values = []
-		for i in range(multi_steps):
-			multi_values += [str(multi_step * i)]
+		multi_bcc_values = []
+		if multi_bcc_steps > 1:
+			multi_bcc_step = multi_step
+			for i in range(multi_bcc_steps):
+				multi_values += [str(multi_bcc_step  * i)]
+			for i in range(multi_bcc_steps * 2 - 1):
+				multi_bcc_values += [str(multi_bcc_step / 2.0  * i)]
+		else:
+			for i in range(multi_steps):
+				multi_values += [str(multi_step * i)]
 		if single_channel_patches > 1:
 			single_channel_step = 255.0 / (single_channel_patches - 1)
 			for i in range(single_channel_patches):
@@ -810,17 +903,22 @@ def get_total_patches(white_patches=None, single_channel_patches=None,
 		if gray_patches > 1:
 			gray_step = 255.0 / (gray_patches - 1)
 			for i in range(gray_patches):
-				if str(gray_step * i) in multi_values:
+				if (str(gray_step * i) in multi_values or
+					str(gray_step * i) in multi_bcc_values):
 					gray_patches -= 1
 	elif gray_patches > 1:
 		white_patches -= 1  # white always in gray patches
 		single_channel_patches_total -= 3  # black always in gray patches
-	else:
+	elif single_channel_patches_total:
 		# black always only once in single channel patches
-		single_channel_patches_total -= 2 
+		single_channel_patches_total -= 2
 	total_patches += max(0, white_patches) + \
 					 max(0, single_channel_patches_total) + \
 					 max(0, gray_patches) + fullspread_patches
+	if black_patches:
+		if gray_patches > 1 or single_channel_patches_total or multi_steps:
+			black_patches -= 1  # black always in other patches
+		total_patches += black_patches
 	return total_patches
 
 
