@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 from codecs import EncodedFile
+from hashlib import md5
 import logging
 import logging.handlers
 import os
@@ -12,9 +13,11 @@ from config import isapp, logdir
 from meta import name as appname
 from safe_print import SafePrinter, safe_print as _safe_print
 from util_io import StringIOu as StringIO
-from util_str import safe_unicode, universal_newlines
+from util_str import safe_str, safe_unicode, universal_newlines
 import config
 
+logging.raiseExceptions = 0
+
 logbuffer = EncodedFile(StringIO(), "UTF-8", errors="replace")
 
 class DummyLogger():
@@ -71,6 +74,27 @@ class Log():
 log = Log()
 
 
+class LogFile():
+	
+	def __init__(self, filename, logdir=logdir):
+		self._logger = get_file_logger(md5(safe_str(filename,
+													"UTF-8")).hexdigest(),
+									   logdir=logdir, filename=filename)
+	
+	def close(self):
+		for handler in reversed(self._logger.handlers):
+			handler.close()
+			self._logger.removeHandler(handler)
+	
+	def flush(self):
+		for handler in self._logger.handlers:
+			handler.flush()
+
+	def write(self, msg):
+		for line in universal_newlines(msg.rstrip()).split("\n"):
+			self._logger.info(line)
+
+
 class SafeLogger(SafePrinter):
 	
 	"""
@@ -96,10 +120,14 @@ safe_log = SafeLogger(print_=False)
 safe_print = SafeLogger()
 
 
-def get_file_logger(name, level=logging.DEBUG, when="midnight", backupCount=0):
+def get_file_logger(name, level=logging.DEBUG, when="midnight", backupCount=0,
+					logdir=logdir, filename=None):
 	logger = logging.getLogger(name)
+	logger.propagate = 0
 	logger.setLevel(level)
-	logfile = os.path.join(logdir, name + ".log")
+	if not filename:
+		filename = name
+	logfile = os.path.join(logdir, filename + ".log")
 	for handler in logger.handlers:
 		if (isinstance(handler, logging.handlers.TimedRotatingFileHandler) and
 			handler.baseFilename == os.path.abspath(logfile)):
