@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 from time import strftime
@@ -37,9 +36,7 @@ def create(report_path, placeholders2data, pack=True, templatename="report"):
 	for placeholder, data in placeholders2data.iteritems():
 		report_html = report_html.replace(placeholder, data)
 	
-	for include in ("base.css", "compare.css", "compare-dark-light.css", 
-					"compare-dark.css", "compare-light.css", 
-					"compare-light-dark.css", "print.css", 
+	for include in ("base.css", "compare.css", "print.css", 
 					"jsapi-packages.js", "jsapi-patches.js", 
 					"compare.constants.js", "compare.variables.js", 
 					"compare.functions.js", "compare.init.js",
@@ -104,7 +101,17 @@ def update(report_path, pack=True):
 			("${PROFILE_WHITEPOINT_NORMALIZED}", 
 			 '"FF_profile_whitepoint_normalized"\s*value="(.+?)"\s\/>', 0),
 			("${SIMULATION_PROFILE}",
-			 'SIMULATION_PROFILE\s*=\s*"(.+?)";', 0),
+			 'SIMULATION_PROFILE\s*=\s*"(.+?)"[;,]', 0),
+			("${BT_1886_GAMMA}",
+			 'BT_1886_GAMMA\s*=\s*(.+?)[;,]', 0),
+			("${BT_1886_GAMMA_TYPE}",
+			 'BT_1886_GAMMA_TYPE\s*=\s*"(.+?)"[;,]', 0),
+			("${WHITEPOINT_SIMULATION}",
+			 'WHITEPOINT_SIMULATION\s*=\s*(.+?)[;,]', 0),
+			("${WHITEPOINT_SIMULATION_RELATIVE}",
+			 'WHITEPOINT_SIMULATION_RELATIVE\s*=\s*(.+?)[;,]', 0),
+			("${DEVICELINK_PROFILE}",
+			 'DEVICELINK_PROFILE\s*=\s*"(.+?)"[;,]', 0),
 			("${TESTCHART}", '"FF_testchart"\s*value="(.+?)"\s\/>', 0),
 			("${ADAPTION}", '"FF_adaption"\s*value="(.+?)"\s\/>', 0),
 			("${DATETIME}", '"FF_datetime"\s*value="(.+?)"\s\/>', 0),
@@ -120,7 +127,10 @@ def update(report_path, pack=True):
 						 "${CAL_ENTRYCOUNT}": "null",
 						 "${CAL_RGBLEVELS}": "null",
 						 "${GRAYSCALE}": "null",
-						 "${BLACKPOINT}": "-1 -1 -1"}
+						 "${BLACKPOINT}": "-1 -1 -1",
+						 "${BT_1886_GAMMA}": "null",
+						 "${WHITEPOINT_SIMULATION}": "false",
+						 "${WHITEPOINT_SIMULATION_RELATIVE}": "false"}
 	
 	for placeholder, pattern, flags in data:
 		result = re.search(pattern, orig_report_html, flags)
