@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 """
@@ -229,6 +228,7 @@ def setup():
 			"pnp.ids",
 			"lang/*.json",
 			"presets/*.icc",
+			"quirk.json",
 			"ref/*.cie",
 			"ref/*.gam",
 			"ref/*.icm",
@@ -243,6 +243,7 @@ def setup():
 			"theme/icons/72x72/*.png",
 			"ti1/*.ti1",
 			"xrc/*.xrc",
+			"linear.cal",
 			"test.cal"
 		] if sys.platform in ("darwin", "win32") and not do_py2app and not 
 		do_py2exe else []
@@ -288,6 +289,7 @@ def setup():
 			(data, [os.path.join(pydir, "beep.wav")]), 
 			(data, [os.path.join(pydir, "camera_shutter.wav")]), 
 			(data, [os.path.join(pydir, "pnp.ids")]), 
+			(data, [os.path.join(pydir, "quirk.json")]), 
 			(os.path.join(data, "lang"), 
 				glob.glob(os.path.join(pydir, "lang", "*.json"))), 
 			(os.path.join(data, "presets"), 
@@ -314,6 +316,7 @@ def setup():
 				glob.glob(os.path.join(pydir, "ti1", "*.ti1"))), 
 			(os.path.join(data, "xrc"), 
 				glob.glob(os.path.join(pydir, "xrc", "*.xrc"))),
+			(data, [os.path.join(pydir, "linear.cal")]),
 			(data, [os.path.join(pydir, "test.cal")])
 		]
 		if sys.platform == "win32":
@@ -328,8 +331,12 @@ def setup():
 					 [os.path.join("scripts", name + "-3DLUT-maker")]),
 					(os.path.join(data, "scripts"), 
 					 [os.path.join("scripts", name + "-apply-profiles")]),
+					(os.path.join(data, "scripts"), 
+					 [os.path.join("scripts", name + "-curve-viewer")]),
 					(os.path.join(data, "scripts"), 
 					 [os.path.join("scripts", name + "-profile-info")]),
+					(os.path.join(data, "scripts"), 
+					 [os.path.join("scripts", name + "-synthprofile")]),
 					(os.path.join(data, "scripts"), 
 					 [os.path.join("scripts", name + "-testchart-editor")])]
 			else:
@@ -434,8 +441,8 @@ def setup():
 			p = sp.Popen([sys.executable, '-c', '''import os
 from distutils.core import setup, Extension
 
-setup(ext_modules = [Extension("%s.RealDisplaySizeMM", sources=%r, 
-							   define_macros=%r, extra_link_args=%r)''' % 
+setup(ext_modules=[Extension("%s.RealDisplaySizeMM", sources=%r, 
+							 define_macros=%r, extra_link_args=%r)])''' % 
 						  (name, sources, macros, link_args)] + sys.argv[1:], 
 						 stdout = sp.PIPE, stderr = sp.STDOUT)
 			lines = []
@@ -554,7 +561,9 @@ setup(ext_modules = [Extension("%s.RealDisplaySizeMM", sources=%r,
 	else:
 		attrs["scripts"] += [os.path.join("scripts", name)]
 		attrs["scripts"] += [os.path.join("scripts", name + "-3DLUT-maker")]
+		attrs["scripts"] += [os.path.join("scripts", name + "-curve-viewer")]
 		attrs["scripts"] += [os.path.join("scripts", name + "-profile-info")]
+		attrs["scripts"] += [os.path.join("scripts", name + "-synthprofile")]
 		attrs["scripts"] += [os.path.join("scripts", name + "-testchart-editor")]
 	
 	if sys.platform != "darwin" and (sys.platform != "win32" or not do_py2exe):
@@ -618,13 +627,31 @@ setup(ext_modules = [Extension("%s.RealDisplaySizeMM", sources=%r,
 			name + (".exe.VC90.manifest" if hasattr(sys, "version_info") and 
 			sys.version_info[:2] >= (2,6) else ".exe.manifest")))
 		attrs["windows"] = [Target(**{
-			"script": os.path.join(pydir, "..", name + ".pyw"),
+			"script": os.path.join(pydir, "..", script),
 			"icon_resources": [(1, os.path.join(pydir, "theme", "icons", 
 												name + ".ico"))],
 			"other_resources": [(24, 1, manifest_xml)],
 			"copyright": u"© %s %s" % (strftime("%Y"), author),
-			"description": name
-		})]
+			"description": desc
+		}) for desc, script in [(name, name + ".pyw"),
+								("Profile Loader",
+								 os.path.join("scripts",
+											  name + "-apply-profiles")),
+								("3DLUT Maker",
+								 os.path.join("scripts",
+											  name + "-3DLUT-maker")),
+								("Curve Viewer",
+								 os.path.join("scripts",
+											  name + "-curve-viewer")),
+								("Profile Info",
+								 os.path.join("scripts",
+											  name + "-profile-info")),
+								("Synthetic Profile Creator",
+								 os.path.join("scripts",
+											  name + "-synthprofile")),
+								("Testchart Editor",
+								 os.path.join("scripts",
+											  name + "-testchart-editor"))]]
 		dist_dir = os.path.join(pydir, "..", "dist", "py2exe.%s-py%s" % 
 								(get_platform(), sys.version[:3]), name + 
 								"-" + version)
@@ -807,6 +834,8 @@ setup(ext_modules = [Extension("%s.RealDisplaySizeMM", sources=%r,
 					"camera_shutter.wav",
 					name + ".desktop",
 					"pnp.ids",
+					"quirk.json",
+					"linear.cal",
 					"test.cal"
 				]:
 					path = os.path.join(data, fname)
