@@ -3,6 +3,7 @@
 import locale
 import os
 import re
+import shutil
 import subprocess as sp
 import sys
 import tempfile
@@ -199,6 +200,22 @@ def listdir_re(path, rex = None):
 	return files
 
 
+def movefile(src, dst, overwrite=True):
+	""" Move a file to another location.
+	
+	dst can be a directory in which case a file with the same basename as src
+	will be created in it.
+	
+	Set overwrite to True to make sure existing files are overwritten.
+
+	"""
+	if os.path.isdir(dst):
+		dst = os.path.join(dst, os.path.basename(src))
+	if os.path.isfile(dst) and overwrite:
+		os.remove(dst)
+	shutil.move(src, dst)
+
+
 def putenvu(name, value):
 	""" Unicode version of os.putenv (also correctly updates os.environ) """
 	if sys.platform == "win32" and isinstance(value, unicode):
@@ -233,7 +250,10 @@ def waccess(path, mode):
 		else:
 			dir = os.path.dirname(path)
 		try:
-			test = tempfile.TemporaryFile(prefix=".", dir=dir)
+			if os.path.isfile(path):
+				test = open(path, "ab")
+			else:
+				test = tempfile.TemporaryFile(prefix=".", dir=dir)
 		except EnvironmentError:
 			return False
 		test.close()
