@@ -1,24 +1,29 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 from decimal import Decimal
 import math
 import os
+import re
 import subprocess as sp
 import sys
 import tempfile
 import traceback
 
-import numpy
+from numpy import interp
 
-from argyll_cgats import cal_to_fake_profile
-from config import fs_enc, get_bitmap_as_icon, getcfg, geticon, setcfg
+from argyll_cgats import cal_to_fake_profile, vcgt_to_cal
+from config import (fs_enc, get_bitmap_as_icon, get_display_profile,
+					get_display_rects, getcfg, geticon, setcfg)
+from log import safe_print
 from meta import name as appname
+from options import debug
 from util_decimal import float2dec
 from util_os import waccess
-from worker import Error, Worker, get_argyll_util, show_result_dialog
+from worker import (Error, Worker, get_argyll_util, make_argyll_compatible_path,
+					show_result_dialog)
 from wxaddons import FileDrop, wx
 from wxenhancedplot import _Numeric
+from wxMeasureFrame import MeasureFrame
 from wxwindows import InfoDialog
 import colormath
 import config
@@ -96,7 +101,7 @@ class CoordinateType(list):
 						  ("SMPTE 240M", -240),
 						  ("L*", -3.0),
 						  ("sRGB", -2.4),
-						  (("Gamma %.2f" % gamma).rstrip("0"), gamma)):
+						  ("Gamma %.2f" % gamma, gamma)):
 			trc.set_trc(exp, self, vmin, vmax)
 			match[(name, exp)] = 0.0
 			count = 0
@@ -170,7 +175,8 @@ class LUTCanvas(plot.PlotCanvas):
 			yLabel = ""
 		
 		detect_increments = False
-		Plot = plot.PolySpline
+		Plot = plot.PolyLine
+		Plot._attributes["width"] = 1
 
 		lines = []
 		linear_points = []
@@ -459,6 +465,52 @@ class LUTFrame(wx.Frame):
 		self.plot_mode_select.SetMaxFontSize(11)
 		self.cbox_sizer.Add(self.plot_mode_select, flag=wx.ALIGN_CENTER_VERTICAL)
 		self.Bind(wx.EVT_CHOICE, self.DrawLUT, id=self.plot_mode_select.GetId())
+
+		self.reload_vcgt_btn = wx.BitmapButton(self.box_panel, -1,
+											 geticon(16, "stock_refresh"),
+											 style=wx.NO_BORDER)
+		self.reload_vcgt_btn.SetBackgroundColour(BGCOLOUR)
+		self.cbox_sizer.Add(self.reload_vcgt_btn, flag=wx.ALIGN_CENTER_VERTICAL |
+													   wx.LEFT, border=20)
+		self.reload_vcgt_btn.Bind(wx.EVT_BUTTON, self.reload_vcgt_handler)
+		self.reload_vcgt_btn.SetToolTipString(
+			lang.getstr("calibration.load_from_display_profile"))
+		self.reload_vcgt_btn.SetBitmapDisabled(geticon(16, "empty"))
+		self.reload_vcgt_btn.Disable()
+		
+		self.apply_bpc_btn = wx.BitmapButton(self.box_panel, -1,
+											 geticon(16, "black_point"),
+											 style=wx.NO_BORDER)
+		self.apply_bpc_btn.SetBackgroundColour(BGCOLOUR)
+		self.cbox_sizer.Add(self.apply_bpc_btn, flag=wx.ALIGN_CENTER_VERTICAL |
+													 wx.LEFT, border=16)
+		self.apply_bpc_btn.Bind(wx.EVT_BUTTON, self.apply_bpc_handler)
+		self.apply_bpc_btn.SetToolTipString(lang.getstr("black_point_compensation"))
+		self.apply_bpc_btn.SetBitmapDisabled(geticon(16, "empty"))
+		self.apply_bpc_btn.Disable()
+
+		self.install_vcgt_btn = wx.BitmapButton(self.box_panel, -1,
+												geticon(16, "install"),
+												style=wx.NO_BORDER)
+		self.install_vcgt_btn.SetBackgroundColour(BGCOLOUR)
+		self.cbox_sizer.Add(self.install_vcgt_btn,
+							flag=wx.ALIGN_CENTER_VERTICAL | wx.LEFT, border=16)
+		self.install_vcgt_btn.Bind(wx.EVT_BUTTON, self.install_vcgt_handler)
+		self.install_vcgt_btn.SetToolTipString(lang.getstr("apply_cal"))
+		self.install_vcgt_btn.SetBitmapDisabled(geticon(16, "empty"))
+		self.install_vcgt_btn.Disable()
+
+		self.save_vcgt_btn = wx.BitmapButton(self.box_panel, -1,
+											 geticon(16, "media-floppy"),
+											 style=wx.NO_BORDER)
+		self.save_vcgt_btn.SetBackgroundColour(BGCOLOUR)
+		self.cbox_sizer.Add(self.save_vcgt_btn, flag=wx.ALIGN_CENTER_VERTICAL |
+													 wx.LEFT, border=16)
+		self.save_vcgt_btn.Bind(wx.EVT_BUTTON, self.SaveFile)
+		self.save_vcgt_btn.SetToolTipString(lang.getstr("save_as") + " " +
+											"(*.cal)")
+		self.save_vcgt_btn.SetBitmapDisabled(geticon(16, "empty"))
+		self.save_vcgt_btn.Disable()
 		
 		self.cbox_sizer.Add((20, 0))
 		
@@ -514,10 +566,28 @@ class LUTFrame(wx.Frame):
 		self.cbox_sizer.Add(self.save_plot_btn, flag=wx.ALIGN_CENTER_VERTICAL |
 													 wx.LEFT, border=16)
 		self.save_plot_btn.Bind(wx.EVT_BUTTON, self.SaveFile)
-		self.save_plot_btn.SetToolTipString(lang.getstr("save_as"))
+		self.save_plot_btn.SetToolTipString(lang.getstr("save_as") + " " +
+											"(*.bmp, *.xbm, *.xpm, *.jpg, *.png)")
 		self.save_plot_btn.SetBitmapDisabled(geticon(16, "empty"))
 		self.save_plot_btn.Disable()
 
+		self.box_sizer.Add((0, 6))
+		self.box_sizer.Add((0, 6))
+		self.box_sizer.Add((0, 6))
+
+		self.box_sizer.Add((0, 0))
+
+		self.show_actual_lut_cb = wx.CheckBox(self.box_panel, -1,
+											  lang.getstr("calibration.show_actual_lut"))
+		self.show_actual_lut_cb.SetForegroundColour(FGCOLOUR)
+		self.show_actual_lut_cb.SetMaxFontSize(11)
+		self.box_sizer.Add(self.show_actual_lut_cb,
+							flag=wx.ALIGN_CENTER | wx.BOTTOM, border=16)
+		self.Bind(wx.EVT_CHECKBOX, self.show_actual_lut_handler,
+				  id=self.show_actual_lut_cb.GetId())
+
+		self.box_sizer.Add((0, 0))
+
 		self.client.canvas.Bind(wx.EVT_MOTION, self.OnMotion)
 		
 		self.droptarget = FileDrop()
@@ -537,6 +607,16 @@ class LUTFrame(wx.Frame):
 		
 		self.Bind(wx.EVT_MOVE, self.OnMove)
 		self.Bind(wx.EVT_SIZE, self.OnSize)
+		
+		self.display_no = -1
+		self.display_rects = get_display_rects()
+	
+	def apply_bpc_handler(self, event):
+		cal = vcgt_to_cal(self.profile)
+		cal.filename = self.profile.fileName or ""
+		cal.apply_bpc(True)
+		self.LoadProfile(cal_to_fake_profile(cal))
+		self.DrawLUT()
 
 	def drop_handler(self, path):
 		"""
@@ -577,12 +657,112 @@ class LUTFrame(wx.Frame):
 				   ok=lang.getstr("ok"), 
 				   bitmap=geticon(32, "dialog-error"))
 	
+	get_display = MeasureFrame.__dict__["get_display"]
+	
+	def install_vcgt_handler(self, event):
+		cwd = self.worker.create_tempdir()
+		if isinstance(cwd, Exception):
+			show_result_dialog(cwd, self)
+		else:
+			cal = os.path.join(cwd, re.sub(r"[\\/:*?\"<>|]+",
+										   "",
+										   make_argyll_compatible_path(
+											   config.get_display_name(
+												   include_geometry=True) or 
+											   "Video LUT")))
+			vcgt_to_cal(self.profile).write(cal)
+			cmd, args = self.worker.prepare_dispwin(cal)
+			if isinstance(cmd, Exception):
+				show_result_dialog(cmd, self)
+			elif cmd:
+				result = self.worker.exec_cmd(cmd, args, capture_output=True, 
+											  skip_scripts=True)
+				if isinstance(result, Exception):
+					show_result_dialog(result, self)
+				elif not result:
+					show_result_dialog(Error("".join(self.worker.errors)),
+									   self)
+			# Important:
+			# Make sure to only delete the temporary cal file we created
+			# (which hasn't an extension, so we can use ext_filter to 
+			# exclude files which should not be deleted)
+			self.worker.wrapup(copy=False, ext_filter=[".app", ".cal", 
+													   ".ccmx", ".ccss",
+													   ".cmd", ".command", 
+													   ".gam", ".gz",
+													   ".icc", ".icm",
+													   ".log",
+													   ".sh", ".ti1",
+													   ".ti3", ".wrl",
+													   ".wrz"])
+	
 	def toggle_clut_handler(self, event):
 		try:
 			self.lookup_tone_response_curves()
 		except Exception, exception:
 			show_result_dialog(exception, self)
 		else:
+			self.trc = None
+			self.DrawLUT()
+
+	def show_actual_lut_handler(self, event):
+		setcfg("lut_viewer.show_actual_lut", 
+			   int(self.show_actual_lut_cb.GetValue()))
+		if hasattr(self, "current_cal"):
+			profile = self.current_cal
+		else:
+			profile = None
+		self.load_lut(profile=profile)
+	
+	def load_lut(self, profile=None):
+		self.current_cal = profile
+		if (getcfg("lut_viewer.show_actual_lut") and
+			(self.worker.argyll_version[0:3] > [1, 1, 0] or
+			 (self.worker.argyll_version[0:3] == [1, 1, 0] and
+			  not "Beta" in self.worker.argyll_version_string)) and
+			not config.get_display_name() in ("Web", "Untethered")):
+			tmp = self.worker.create_tempdir()
+			if isinstance(tmp, Exception):
+				show_result_dialog(tmp, self)
+				return
+			cmd, args = (get_argyll_util("dispwin"), 
+						 ["-d" + self.worker.get_display(), "-s", 
+						  os.path.join(tmp, 
+									   re.sub(r"[\\/:*?\"<>|]+",
+											  "",
+											  make_argyll_compatible_path(
+												config.get_display_name(
+													include_geometry=True) or 
+												"Video LUT")))])
+			result = self.worker.exec_cmd(cmd, args, capture_output=True, 
+										  skip_scripts=True, silent=not __name__ == "__main__")
+			if not isinstance(result, Exception) and result:
+				profile = cal_to_fake_profile(args[-1])
+			else:
+				if isinstance(result, Exception):
+					safe_print(result)
+			# Important: lut_viewer_load_lut is called after measurements,
+			# so make sure to only delete the temporary cal file we created
+			# (which hasn't an extension, so we can use ext_filter to 
+			# exclude files which should not be deleted)
+			self.worker.wrapup(copy=False, ext_filter=[".app", ".cal", 
+													   ".ccmx", ".ccss",
+													   ".cmd", ".command", 
+													   ".gam", ".gz",
+													   ".icc", ".icm",
+													   ".log",
+													   ".sh", ".ti1",
+													   ".ti3", ".wrl",
+													   ".wrz"])
+		if profile and (profile.is_loaded or not profile.fileName or 
+						os.path.isfile(profile.fileName)):
+			if not self.profile or \
+			   self.profile.fileName != profile.fileName or \
+			   not self.profile.isSame(profile):
+				self.LoadProfile(profile)
+				self.DrawLUT()
+		else:
+			self.LoadProfile(None)
 			self.DrawLUT()
 	
 	def lookup_tone_response_curves(self, intent="r"):
@@ -629,8 +809,8 @@ class LUTFrame(wx.Frame):
 				for i in xrange(size):
 					x.append(i / (size - 1.0) * 255)
 					y.append(colormath.Lab2XYZ(i / (size - 1.0) * 100, 0, 0)[1] * 100)
-				yi = numpy.interp(x, xp, yp)
-				xi = numpy.interp(y, yi, x)
+				yi = interp(x, xp, yp)
+				xi = interp(y, yi, x)
 				setattr(self, sig, CoordinateType())
 				for v, Y in zip(xi, y):
 					if Y <= yp[0]:
@@ -745,6 +925,40 @@ class LUTFrame(wx.Frame):
 				elif i == 2:
 					self.bTRC.append([y, v])
 
+	def move_handler(self, event):
+		if not self.IsShownOnScreen():
+			return
+		display_no, geometry, client_area = self.get_display()
+		if display_no != self.display_no:
+			self.display_no = display_no
+			# Translate from wx display index to Argyll display index
+			geometry = "%i, %i, %ix%i" % tuple(geometry)
+			for i, display in enumerate(getcfg("displays").split(os.pathsep)):
+				if display.find("@ " + geometry) > -1:
+					if debug:
+						safe_print("[D] Found display %s at index %i" % 
+								   (geometry, i))
+					# Save Argyll display index to configuration
+					setcfg("display.number", i + 1)
+					self.load_lut(get_display_profile(i))
+					break
+		event.Skip()
+	
+	def reload_vcgt_handler(self, event):
+		cmd, args = self.worker.prepare_dispwin(True)
+		if isinstance(cmd, Exception):
+			show_result_dialog(cmd, self)
+		elif cmd:
+			result = self.worker.exec_cmd(cmd, args, capture_output=True, 
+										  skip_scripts=True)
+			if isinstance(result, Exception):
+				show_result_dialog(result, self)
+			elif not result:
+				show_result_dialog(Error("".join(self.worker.errors)),
+								   self)
+			else:
+				self.load_lut(get_display_profile())
+
 	def LoadProfile(self, profile):
 		if not profile:
 			profile = ICCP.ICCProfile()
@@ -844,25 +1058,18 @@ class LUTFrame(wx.Frame):
 				if isinstance(self.rTRC, ICCP.CurveType):
 					self.trc = ICCP.CurveType()
 					for i in xrange(len(self.rTRC)):
-						#self.trc.append(int(round((self.rTRC[i] +
-												   #self.gTRC[i] +
-												   #self.bTRC[i]) / 3.0)))
-						self.trc.append(sorted([self.rTRC[i][0],
-												self.gTRC[i][0],
-												self.bTRC[i][0]])[1])
+						self.trc.append(int(round((self.rTRC[i] +
+												   self.gTRC[i] +
+												   self.bTRC[i]) / 3.0)))
 				else:
 					self.trc = CoordinateType()
 					for i in xrange(len(self.rTRC)):
-						#self.trc.append([(self.rTRC[i][0] +
-										  #self.gTRC[i][0] +
-										  #self.bTRC[i][0]) / 3.0,
-										 #(self.rTRC[i][1] +
-										  #self.gTRC[i][1] +
-										  #self.bTRC[i][1]) / 3.0])
-						coord = sorted([self.rTRC[i],
-										self.gTRC[i],
-										self.bTRC[i]])
-						self.trc.append([coord[1][0], coord[1][1]])
+						self.trc.append([(self.rTRC[i][0] +
+										  self.gTRC[i][0] +
+										  self.bTRC[i][0]) / 3.0,
+										 (self.rTRC[i][1] +
+										  self.gTRC[i][1] +
+										  self.bTRC[i][1]) / 3.0])
 			if getattr(self, "trc", None):
 				transfer_function = self.trc.get_transfer_function(slice=(0.00, 1.00))
 			#if "R" in colorants and "G" in colorants and "B" in colorants:
@@ -961,7 +1168,28 @@ class LUTFrame(wx.Frame):
 							  isinstance(self.profile.tags.get("gTRC"), ICCP.CurveType) and
 							  isinstance(self.profile.tags.get("bTRC"), ICCP.CurveType))
 		self.save_plot_btn.Enable(bool(curves))
+		if hasattr(self, "reload_vcgt_btn"):
+			self.reload_vcgt_btn.Enable(not(self.plot_mode_select.GetSelection()) and
+									    bool(self.profile))
+			self.reload_vcgt_btn.Show(not(self.plot_mode_select.GetSelection()))
+		if hasattr(self, "apply_bpc_btn"):
+			enable_bpc = (not(self.plot_mode_select.GetSelection()) and
+						  bool(self.profile) and "vcgt" in self.profile.tags)
+			if enable_bpc:
+				values = self.profile.tags.vcgt.getNormalizedValues()
+			self.apply_bpc_btn.Enable(enable_bpc and values[0] != (0, 0, 0))
+			self.apply_bpc_btn.Show(not(self.plot_mode_select.GetSelection()))
+		if hasattr(self, "install_vcgt_btn"):
+			self.install_vcgt_btn.Enable(not(self.plot_mode_select.GetSelection()) and
+										 bool(self.profile) and "vcgt" in self.profile.tags)
+			self.install_vcgt_btn.Show(not(self.plot_mode_select.GetSelection()))
+		if hasattr(self, "save_vcgt_btn"):
+			self.save_vcgt_btn.Enable(not(self.plot_mode_select.GetSelection()) and
+									  bool(self.profile) and "vcgt" in self.profile.tags)
+			self.save_vcgt_btn.Show(not(self.plot_mode_select.GetSelection()))
 		self.show_as_L.GetContainingSizer().Layout()
+		if hasattr(self, "show_actual_lut_cb"):
+			self.show_actual_lut_cb.Show(self.plot_mode_select.GetSelection() == 0)
 		if hasattr(self, "cbox_sizer"):
 			self.cbox_sizer.Layout()
 		if hasattr(self, "box_sizer"):
@@ -985,6 +1213,12 @@ class LUTFrame(wx.Frame):
 		self.Thaw()
 		wx.CallLater(125, self.UpdatePointLabel, pointXY)
 
+	def OnClose(self, event):
+		config.writecfg()
+		if self.worker.tempdir and os.path.isdir(self.worker.tempdir):
+			self.worker.wrapup(False)
+		wx.GetApp().ExitMainLoop()
+
 	def OnMotion(self, event):
 		if isinstance(event, wx.MouseEvent):
 			xy = self.client._getXY(event)
@@ -1026,13 +1260,17 @@ class LUTFrame(wx.Frame):
 		fileName = " ".join([self.plot_mode_select.GetStringSelection(),
 							 os.path.basename(os.path.splitext(self.profile.fileName or
 															   lang.getstr("unnamed"))[0])])
-		extensions = {
-			"bmp": wx.BITMAP_TYPE_BMP,       # Save a Windows bitmap file.
-			"xbm": wx.BITMAP_TYPE_XBM,       # Save an X bitmap file.
-			"xpm": wx.BITMAP_TYPE_XPM,       # Save an XPM bitmap file.
-			"jpg": wx.BITMAP_TYPE_JPEG,      # Save a JPG file.
-			"png": wx.BITMAP_TYPE_PNG,       # Save a PNG file.
-			}
+		if (event and hasattr(self, "save_vcgt_btn") and
+			event.GetId() == self.save_vcgt_btn.GetId()):
+			extensions = {"cal": 1}
+		else:
+			extensions = {
+				"bmp": wx.BITMAP_TYPE_BMP,       # Save a Windows bitmap file.
+				"xbm": wx.BITMAP_TYPE_XBM,       # Save an X bitmap file.
+				"xpm": wx.BITMAP_TYPE_XPM,       # Save an XPM bitmap file.
+				"jpg": wx.BITMAP_TYPE_JPEG,      # Save a JPG file.
+				"png": wx.BITMAP_TYPE_PNG,       # Save a PNG file.
+				}
 
 		fType = fileName[-3:].lower()
 		dlg1 = None
@@ -1047,15 +1285,16 @@ class LUTFrame(wx.Frame):
 				dlg1 = wx.FileDialog(
 					self, 
 					lang.getstr("save_as"), ".", fileName,
-					"PNG (*.png)|*.png|BMP (*.bmp)|*.bmp|XBM (*.xbm)|*.xbm|XPM (*.xpm)|*.xpm|JPG (*.jpg)|*.jpg",
+					"|".join(["%s (*.%s)|*.%s" % (ext.upper(), ext, ext)
+							  for ext in extensions.keys()]),
 					wx.SAVE|wx.OVERWRITE_PROMPT
 					)
 
 			if dlg1.ShowModal() == wx.ID_OK:
 				fileName = dlg1.GetPath()
-				if not waccess(os.path.dirname(fileName), os.W_OK):
+				if not waccess(fileName, os.W_OK):
 					show_result_dialog(Error(lang.getstr("error.access_denied.write",
-														 os.path.dirname(fileName))), self)
+														 fileName)), self)
 					return
 				fType = fileName[-3:].lower()
 			else:                      # exit without saving
@@ -1066,7 +1305,12 @@ class LUTFrame(wx.Frame):
 			dlg1.Destroy()
 
 		# Save Bitmap
-		res= self.client._Buffer.SaveFile(fileName, extensions.get(fType, ".png"))
+		if (event and hasattr(self, "save_vcgt_btn") and
+			event.GetId() == self.save_vcgt_btn.GetId()):
+			res = vcgt_to_cal(self.profile)
+			res.write(fileName)
+		else:
+			res= self.client._Buffer.SaveFile(fileName, extensions.get(fType, ".png"))
 		return res
 	
 	def UpdatePointLabel(self, xy):
@@ -1113,7 +1357,7 @@ class LUTFrame(wx.Frame):
 					pointXY = pointXY[1], pointXY[0]
 				else:
 					joiner = u" \u2192 "
-					format = "%i", "%.2f"
+					format = "%.2f", "%.2f"
 					axis_y = 255.0
 					legend[0] += " " + joiner.join([format[i] % point
 													for i, point in
@@ -1133,13 +1377,27 @@ class LUTFrame(wx.Frame):
 							"scaledXY": scaledXY}
 				# Pass dict to update the point label
 				self.client.UpdatePointLabel(mDataDict)
+	
+	def update_controls(self):
+		self.show_actual_lut_cb.Enable((self.worker.argyll_version[0:3] > [1, 1, 0] or
+										(self.worker.argyll_version[0:3] == [1, 1, 0] and
+										 not "Beta" in self.worker.argyll_version_string)) and
+									   not config.get_display_name() in
+									   ("Web", "Untethered"))
+		self.show_actual_lut_cb.SetValue(bool(getcfg("lut_viewer.show_actual_lut")) and
+										 not config.get_display_name() in
+										 ("Web", "Untethered"))
+	
+	@property
+	def worker(self):
+		return self.client.worker
 
 
 class LUTViewer(wx.App):
 
 	def OnInit(self):
 		self.frame = LUTFrame(None, -1)
-		self.frame.Show(True)
+		self.frame.Bind(wx.EVT_CLOSE, self.frame.OnClose, self.frame)
 		return True
 
 
@@ -1148,8 +1406,15 @@ def main(profile=None):
 	lang.init()
 	lang.update_defaults()
 	app = LUTViewer(0)
-	app.frame.LoadProfile(profile or ICCP.get_display_profile())
-	app.frame.DrawLUT()
+	app.frame.worker.enumerate_displays_and_ports(check_lut_access=False,
+												  enumerate_ports=False)
+	app.frame.Bind(wx.EVT_MOVE, app.frame.move_handler, app.frame)
+	app.frame.Show(True)
+	app.frame.update_controls()
+	if profile:
+		app.frame.drop_handler(profile)
+	else:
+		app.frame.load_lut(get_display_profile())
 	app.MainLoop()
 
 if __name__ == '__main__':
