@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 import re
@@ -9,10 +8,11 @@ import sys
 import tempfile
 
 from config import (defaults, fs_enc, get_bitmap_as_icon, get_data_path,
-					getbitmap, getcfg, geticon, setcfg, writecfg)
+					get_display_profile, getbitmap, getcfg, geticon, setcfg,
+					writecfg)
 from meta import name as appname
 from ordereddict import OrderedDict
-from util_str import universal_newlines, wrap
+from util_str import safe_unicode, universal_newlines, wrap
 from worker import (Error, check_set_argyll_bin, get_argyll_util,
 					show_result_dialog)
 from wxaddons import get_platform_window_decoration_size, wx
@@ -28,6 +28,7 @@ import ICCProfile as ICCP
 
 BGCOLOUR = "#333333"
 FGCOLOUR = "#999999"
+TEXTCOLOUR = "#333333"
 
 if sys.platform == "darwin":
 	FONTSIZE_LARGE = 11
@@ -430,12 +431,12 @@ class GamutCanvas(LUTCanvas):
 		
 		if not profiles:
 			profiles = [ICCP.ICCProfile(get_data_path("ref/sRGB.icm")),
-						ICCP.get_display_profile()]
+						get_display_profile()]
 		for i, profile in enumerate(profiles):
 			if profile_no is not None and i != profile_no:
 				continue
 
-			if not profile:
+			if not profile or profile.profileClass == "link":
 				self.set_pcs_data(i)
 				self.profiles[i] = ""
 				continue
@@ -505,9 +506,13 @@ class GamutCanvas(LUTCanvas):
 
 			# Lookup RGB -> XYZ values through profile using xicclu
 			stderr = tempfile.SpooledTemporaryFile()
-			p = sp.Popen([xicclu, "-ff", "-i" + intent, "-px", "profile.icc"], 
-						 stdin=sp.PIPE, stdout=sp.PIPE, stderr=stderr, 
-						 cwd=cwd.encode(fs_enc), startupinfo=startupinfo)
+			try:
+				p = sp.Popen([xicclu, "-ff", "-i" + intent, "-px", "profile.icc"], 
+							 stdin=sp.PIPE, stdout=sp.PIPE, stderr=stderr, 
+							 cwd=cwd.encode(fs_enc), startupinfo=startupinfo)
+			except Exception, exception:
+				raise Error("\n".join([safe_unicode(v) for v in (xicclu,
+																 exception)]))
 			self.worker.subprocess = p
 			if p.poll() not in (0, None):
 				stderr.seek(0)
@@ -823,7 +828,7 @@ class ProfileInfoFrame(LUTFrame):
 		self.SetSizer(self.sizer)
 		
 		self.title_panel = BitmapBackgroundPanelText(self)
-		self.title_panel.SetForegroundColour(BGCOLOUR)
+		self.title_panel.SetForegroundColour(TEXTCOLOUR)
 		self.title_panel.SetBitmap(getbitmap("theme/gradient"))
 		self.sizer.Add(self.title_panel, flag=wx.EXPAND)
 		
@@ -958,7 +963,7 @@ class ProfileInfoFrame(LUTFrame):
 		self.grid.SetCellHighlightPenWidth(0)
 		self.grid.SetCellHighlightROPenWidth(0)
 		self.grid.SetDefaultCellBackgroundColour(gridbgcolor)
-		self.grid.SetDefaultCellTextColour(BGCOLOUR)
+		self.grid.SetDefaultCellTextColour(TEXTCOLOUR)
 		font = wx.Font(FONTSIZE_MEDIUM, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, 
 					   wx.FONTWEIGHT_NORMAL)
 		self.grid.SetDefaultCellFont(font)
@@ -1221,7 +1226,6 @@ class ProfileInfoFrame(LUTFrame):
 		wx.CallAfter(self.resize_grid)
 	
 	def OnSize(self, event=None):
-		self.title_txt.Refresh()
 		if self.IsShownOnScreen() and self.IsMaximized():
 			self.splitter.AdjustLayout()
 		self.splitter.Refresh()
@@ -1376,8 +1380,10 @@ class ProfileInfoFrame(LUTFrame):
 			self.client.Parent.Thaw()
 	
 	def resize_grid(self, event=None):
-		self.grid.SetColSize(1, self.grid.GetSize()[0] -
-								self.grid.GetColSize(0) - wx.SystemSettings_GetMetric(wx.SYS_VSCROLL_X))
+		self.grid.SetColSize(1, max(self.grid.GetSize()[0] -
+									self.grid.GetColSize(0) -
+									wx.SystemSettings_GetMetric(wx.SYS_VSCROLL_X),
+									0))
 		self.grid.SetMargins(0 - wx.SystemSettings_GetMetric(wx.SYS_VSCROLL_X),
 							 0 - wx.SystemSettings_GetMetric(wx.SYS_HSCROLL_Y))
 		self.grid.ForceRefresh()
@@ -1395,6 +1401,7 @@ class ProfileInfoFrame(LUTFrame):
 class ProfileInfoViewer(wx.App):
 
 	def OnInit(self):
+		check_set_argyll_bin()
 		self.frame = ProfileInfoFrame(None, -1)
 		return True
 
@@ -1404,7 +1411,7 @@ def main(profile=None):
 	lang.init()
 	lang.update_defaults()
 	app = ProfileInfoViewer(0)
-	app.frame.LoadProfile(profile or ICCP.get_display_profile())
+	app.frame.LoadProfile(profile or get_display_profile())
 	app.frame.Show(True)
 	app.MainLoop()
 	writecfg()
