@@ -1,6 +1,6 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+from time import gmtime, strftime, time
 import math
 import os
 import sys
@@ -31,6 +31,7 @@ try:
 	import wx.lib.agw.floatspin as floatspin
 except ImportError:
 	import floatspin
+import wx.lib.filebrowsebutton as filebrowse
 
 numpad_keycodes = [wx.WXK_NUMPAD0,
 				   wx.WXK_NUMPAD1,
@@ -366,13 +367,16 @@ class BitmapBackgroundBitmapButton(wx.BitmapButton):
 
 class BitmapBackgroundPanel(wx.Panel):
 	
-	""" A panel with a background bitmap and text label """
+	""" A panel with a background bitmap """
 
 	def __init__(self, *args, **kwargs):
 		wx.Panel.__init__(self, *args, **kwargs)
-		self._bitmap = getbitmap("empty")
+		self._bitmap = None
+		self.scalebitmap = (True, False)
+		self.scalequality = wx.IMAGE_QUALITY_NORMAL
 		self.SetBackgroundStyle(wx.BG_STYLE_CUSTOM)
-		self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
+		self.Bind(wx.EVT_PAINT, self.OnPaint)
+		self.Bind(wx.EVT_SIZE, self.OnSize)
 	
 	def GetBitmap(self):
 		return self._bitmap
@@ -380,21 +384,32 @@ class BitmapBackgroundPanel(wx.Panel):
 	def SetBitmap(self, bitmap):
 		self._bitmap = bitmap
 
-	def OnEraseBackground(self, event):
-		dc = event.GetDC()
-		if not dc:
-			dc = wx.ClientDC(self)
-			rect = self.GetUpdateRegion().GetBox()
-			dc.SetClippingRect(rect)
-		dc.Clear()
-		self._drawbitmap(dc)
+	def OnPaint(self, event):
+		dc = wx.BufferedPaintDC(self)
+		self._draw(dc)
+
+	def OnSize(self,event):
+		self.Refresh()
 	
-	def _drawbitmap(self, dc):
-		dc.DrawBitmap(self._bitmap, 0, 0)
-		pen = wx.Pen(wx.Colour(0x66, 0x66, 0x66), 1, wx.SOLID)
-		pen.SetCap(wx.CAP_BUTT)
-		dc.SetPen(pen)
-		dc.DrawLine(0, self.GetSize()[1] - 1, self.GetSize()[0], self.GetSize()[1] - 1)
+	def _draw(self, dc):
+		bbr = wx.Brush(self.GetBackgroundColour(), wx.SOLID)
+		dc.SetBackground(bbr)
+		dc.SetBackgroundMode(wx.SOLID)
+		dc.Clear()
+		dc.SetTextForeground(self.GetForegroundColour())
+		dc.SetTextBackground(self.GetBackgroundColour())
+		bmp = self._bitmap
+		if bmp:
+			if True in self.scalebitmap:
+				img = bmp.ConvertToImage()
+				bmp = wx.BitmapFromImage(img.Rescale(self.GetSize()[0]
+													 if self.scalebitmap[0]
+													 else img.GetSize()[0],
+													 self.GetSize()[1]
+													 if self.scalebitmap[1]
+													 else img.GetSize()[1],
+													 quality=self.scalequality))
+			dc.DrawBitmap(bmp, 0, 0)
 
 
 class BitmapBackgroundPanelText(BitmapBackgroundPanel):
@@ -404,8 +419,6 @@ class BitmapBackgroundPanelText(BitmapBackgroundPanel):
 	def __init__(self, *args, **kwargs):
 		BitmapBackgroundPanel.__init__(self, *args, **kwargs)
 		self._label = ""
-		self.Unbind(wx.EVT_ERASE_BACKGROUND)
-		self.Bind(wx.EVT_PAINT, self.OnPaint)
 	
 	def _set_font(self, dc):
 		try:
@@ -422,10 +435,13 @@ class BitmapBackgroundPanelText(BitmapBackgroundPanel):
 	
 	def SetLabel(self, label):
 		self._label = label
-
-	def OnPaint(self, dc):
-		dc = wx.PaintDC(self)
-		self._drawbitmap(dc)
+	
+	def _draw(self, dc):
+		BitmapBackgroundPanel._draw(self, dc)
+		pen = wx.Pen(wx.Colour(0x66, 0x66, 0x66), 1, wx.SOLID)
+		pen.SetCap(wx.CAP_BUTT)
+		dc.SetPen(pen)
+		dc.DrawLine(0, self.GetSize()[1] - 1, self.GetSize()[0], self.GetSize()[1] - 1)
 		dc = self._set_font(dc)
 		w1, h1 = self.GetTextExtent(self.GetLabel())
 		w2, h2 = dc.GetTextExtent(self.GetLabel())
@@ -484,6 +500,143 @@ class ConfirmDialog(BaseInteractiveDialog):
 		self.EndModal(id)
 
 
+class FileBrowseBitmapButtonWithChoiceHistory(filebrowse.FileBrowseButtonWithHistory):
+
+	def __init__(self, *arguments, **namedarguments):
+		self.history = namedarguments.get("history") or []
+		if 'history' in namedarguments:
+			del namedarguments["history"]
+
+		self.historyCallBack = None
+		if callable(self.history):
+			self.historyCallBack = self.history
+			self.history = []
+		name = namedarguments.get('name', 'fileBrowseButtonWithHistory')
+		if 'name' in namedarguments:
+			del namedarguments['name']
+		filebrowse.FileBrowseButton.__init__(self, *arguments, **namedarguments)
+		self.SetName(name)
+	
+	def Disable(self):
+		self.Enable(False)
+	
+	def Enable(self, enable=True):
+		self.textControl.Enable(enable and self.history != [""])
+		self.browseButton.Enable(enable)
+
+	def GetValue(self):
+		"""
+		retrieve current value of text control
+		"""
+		if self.textControl.GetSelection() > -1:
+			return self.history[self.textControl.GetSelection()]
+		return ""
+	
+	GetPath = GetValue
+
+	def OnChanged(self, evt):
+		self.textControl.SetToolTipString(self.history[self.textControl.GetSelection()])
+		if self.callCallback and self.changeCallback:
+			self.changeCallback(evt)
+
+	def SetBackgroundColour(self,color):
+		wx.Panel.SetBackgroundColour(self,color)
+
+	def SetHistory(self, value=(), selectionIndex=None, control=None):
+		"""Set the current history list"""
+		if control is None:
+			control = self.textControl
+		if self.history == value:
+			return
+		index = control.GetSelection()
+		if self.history and index > -1:
+			tempValue = self.history[index]
+		else:
+			tempValue = None
+		self.history = value
+		control.Clear()
+		for path in value:
+			control.Append(os.path.basename(path))
+		if tempValue:
+			self.history.append(tempValue)
+			control.Append(os.path.basename(tempValue))
+		self.setupControl(selectionIndex, control)
+	
+	def SetMaxFontSize(self, pointsize=11):
+		self.textControl.SetMaxFontSize(pointsize)
+
+	def SetValue(self, value, callBack=1, clear_on_empty_value=False):
+		if not value:
+			if clear_on_empty_value and self.history:
+				index = self.textControl.GetSelection()
+				if index > -1:
+					self.history.pop(index)
+					self.textControl.Delete(index)
+		if not value in self.history:
+			self.history.append(value)
+			self.textControl.Append(os.path.basename(value))
+		self.setupControl(self.history.index(value))
+		if callBack:
+			self.changeCallback(CustomEvent(wx.EVT_CHOICE.typeId, 
+											self.textControl))
+	
+	def SetPath(self, path):
+		self.SetValue(path, 0, clear_on_empty_value=True)
+
+	def createBrowseButton(self):
+		"""Create the browse-button control"""
+		button = wx.BitmapButton(self, -1, geticon(16, "document-open"), 
+								 style=wx.NO_BORDER)
+		button.SetBitmapDisabled(geticon(16, "empty"))
+		button.SetToolTipString(self.toolTip)
+		button.Bind(wx.EVT_BUTTON, self.OnBrowse)
+		return button
+
+	def createDialog(self, parent, id, pos, size, style, name=None):
+		"""Setup the graphic representation of the dialog"""
+		wx.Panel.__init__ (self, parent, id, pos, size, style)
+		self.SetMinSize(size) # play nice with sizers
+
+		box = wx.BoxSizer(wx.HORIZONTAL)
+
+		self.textControl = self.createTextControl()
+		if self.history:
+			history = self.history
+			self.history = []
+			self.SetHistory(history)
+		box.Add(self.textControl, 1, wx.ALIGN_CENTER_VERTICAL | wx.TOP |
+									 wx.BOTTOM, 4)
+
+		self.browseButton = self.createBrowseButton()
+		box.Add(self.browseButton, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 8)
+
+		self.SetAutoLayout(True)
+		self.SetSizer(box)
+		self.Layout()
+
+	def createLabel(self):
+		return (0, 0)
+
+	def createTextControl(self):
+		"""Create the text control"""
+		textControl = wx.Choice(self, -1)
+		textControl.Bind(wx.EVT_SET_FOCUS, self.OnSetFocus)
+		if self.changeCallback:
+			textControl.Bind(wx.EVT_CHOICE, self.OnChanged)
+		return textControl
+	
+	def setupControl(self, selectionIndex=None, control=None):
+		if control is None:
+			control = self.textControl
+		if selectionIndex is None:
+			selectionIndex = len(self.history) - 1
+		control.SetSelection(selectionIndex)
+		toolTip = (self.history[selectionIndex] if selectionIndex > -1 else
+				   self.toolTip)
+		control.SetToolTipString(toolTip)
+		control.Enable(control.Enabled and self.history != [""])
+
+
 class FileDrop(_FileDrop):
 	
 	def __init__(self, *args, **kwargs):
@@ -708,9 +861,9 @@ class LogWindow(InvincibleFrame):
 		path = dlg.GetPath()
 		dlg.Destroy()
 		if result == wx.ID_OK:
-			if not waccess(os.path.dirname(path), os.W_OK):
+			if not waccess(path, os.W_OK):
 				InfoDialog(self, msg=lang.getstr("error.access_denied.write",
-												 os.path.dirname(path)),
+												 path),
 						   ok=lang.getstr("ok"),
 						   bitmap=geticon(32, "dialog-error"))
 				return
@@ -749,15 +902,16 @@ class LogWindow(InvincibleFrame):
 			event.Skip()
 
 
-class ProgressDialog(wx.ProgressDialog):
+class ProgressDialog(wx.Dialog):
 	
 	""" A progress dialog. """
 	
 	def __init__(self, title=appname, msg="", maximum=100, parent=None, style=None, 
-				 handler=None, keyhandler=None, start_timer=True, pos=None):
+				 handler=None, keyhandler=None, start_timer=True, pos=None,
+				 pauseable=False):
 		if style is None:
 			style = wx.PD_APP_MODAL | wx.PD_ELAPSED_TIME | wx.PD_CAN_ABORT | wx.PD_SMOOTH
-		wx.ProgressDialog.__init__(self, title, "", maximum, parent=parent, style=style)
+		wx.Dialog.__init__(self, parent, wx.ID_ANY, title)
 		self.SetIcons(config.get_icon_bundle([256, 48, 32, 16], appname))
 		self.Bind(wx.EVT_CLOSE, self.OnClose, self)
 		if not pos:
@@ -765,6 +919,56 @@ class ProgressDialog(wx.ProgressDialog):
 		self.timer = wx.Timer(self)
 		self.Bind(wx.EVT_TIMER, handler or self.OnTimer, self.timer)
 		
+		self.keepGoing = True
+		self.skip = False
+		self.paused = False
+
+		margin = 12
+
+		self.sizer0 = wx.BoxSizer(wx.VERTICAL)
+		self.SetSizer(self.sizer0)
+		self.sizer1 = wx.BoxSizer(wx.VERTICAL)
+		self.sizer2 = wx.BoxSizer(wx.HORIZONTAL)
+		self.sizer0.Add(self.sizer1, flag = wx.ALIGN_LEFT | wx.TOP | 
+		   wx.RIGHT | wx.LEFT, border = margin)
+		self.sizer0.Add(self.sizer2, flag = wx.ALIGN_RIGHT | wx.ALL, 
+		   border = margin)
+
+		self.msg = wx.StaticText(self, -1, "")
+		self.sizer1.Add(self.msg, flag=wx.EXPAND | wx.BOTTOM, border=margin)
+		
+		gauge_style = wx.GA_HORIZONTAL
+		if style & wx.PD_SMOOTH:
+			gauge_style |= wx.GA_SMOOTH
+		
+		self.gauge = wx.Gauge(self, wx.ID_ANY, range=maximum, style=gauge_style)
+		self.sizer1.Add(self.gauge, flag=wx.EXPAND | wx.BOTTOM, border=margin)
+		
+		if style & wx.PD_ELAPSED_TIME:
+			self.sizer3 = wx.BoxSizer(wx.HORIZONTAL)
+			self.sizer1.Add(self.sizer3, flag=wx.ALIGN_CENTER)
+			self.sizer3.Add(wx.StaticText(self, -1,
+										  lang.getstr("elapsed_time")))
+			self.time = time()
+			self.elapsed_time = wx.StaticText(self, -1, "")
+			self.elapsed_time_handler(None)
+			self.sizer3.Add(self.elapsed_time, flag=wx.LEFT, border=margin)
+			self.elapsed_timer = wx.Timer(self)
+			self.Bind(wx.EVT_TIMER, self.elapsed_time_handler,
+					  self.elapsed_timer)
+
+		if style & wx.PD_CAN_ABORT:
+			self.cancel = wx.Button(self, wx.ID_ANY, lang.getstr("cancel"))
+			self.sizer2.Add(self.cancel)
+			self.Bind(wx.EVT_BUTTON, self.OnClose, id=self.cancel.GetId())
+
+		self.pause_continue = wx.Button(self, wx.ID_ANY, lang.getstr("pause"))
+		self.sizer2.Prepend((margin, margin))
+		self.sizer2.Prepend(self.pause_continue)
+		self.Bind(wx.EVT_BUTTON, self.pause_continue_handler,
+				  id=self.pause_continue.GetId())
+		self.pause_continue.Show(pauseable)
+		
 		# Use an accelerator table for 0-9, a-z, numpad
 		keycodes = range(48, 58) + range(97, 123) + numpad_keycodes
 		self.id_to_keycode = {}
@@ -776,30 +980,14 @@ class ProgressDialog(wx.ProgressDialog):
 			accels += [(wx.ACCEL_NORMAL, keycode, id)]
 		self.SetAcceleratorTable(wx.AcceleratorTable(accels))
 		
-		# custom localization
-		self.msg = None
-		for child in self.GetChildren():
-			if isinstance(child, wx.Button):
-				child.Label = lang.getstr("cancel")
-			elif isinstance(child, wx.StaticText):
-				if "Elapsed time" in child.Label:
-					child.Label = lang.getstr("elapsed_time").replace(" ", u"\xa0")
-				elif not child.Label:
-					self.msg = child
-					child.SetWindowStyle(wx.ST_NO_AUTORESIZE)
-		
-		if self.msg:
-			text_extent = self.msg.GetTextExtent("E")
-			w, h = (text_extent[0] * 80, 
-					text_extent[1] * 4)
-			self.msg.SetMinSize((w, h))
-			self.msg.SetSize((w, h))
+		text_extent = self.msg.GetTextExtent("E")
+		w, h = (text_extent[0] * 80, 
+				text_extent[1] * 4)
+		self.msg.SetMinSize((w, h))
+		self.msg.SetSize((w, h))
 		self.Fit()
 		self.SetMinSize(self.GetSize())
-		if self.msg:
-			self.msg.SetLabel(msg)
-		else:
-			self.Pulse(msg)
+		self.msg.SetLabel(msg.replace("&", "&&"))
 		
 		self.Bind(wx.EVT_WINDOW_DESTROY, self.OnDestroy, self)
 		
@@ -823,11 +1011,19 @@ class ProgressDialog(wx.ProgressDialog):
 		
 		if start_timer:
 			self.start_timer()
+		
+		self.Show()
+		if style & wx.PD_APP_MODAL:
+			self.MakeModal()
 	
 	def OnClose(self, event):
+		self.keepGoing = False
+		if hasattr(self, "cancel"):
+			self.cancel.Disable()
+		self.pause_continue.Disable()
 		if not self.timer.IsRunning():
 			self.Destroy()
-		else:
+		elif self.gauge.GetValue() == self.gauge.GetRange():
 			event.Skip()
 	
 	def OnDestroy(self, event):
@@ -846,31 +1042,69 @@ class ProgressDialog(wx.ProgressDialog):
 				setcfg("position.progress.y", y)
 	
 	def OnTimer(self, event):
-		if getattr(self, "keepGoing", True):
-			if not hasattr(self, "i"):
-				self.i = 0
-			if self.i < 100:
-				self.i += 1
-				if self.i == 100:
-					self.stop_timer()
-				self.keepGoing, self.skip = self.Update(self.i)
-				if self.i == 100:
-					self.Destroy()
-		if not self.keepGoing:
+		if self.keepGoing:
+			if self.gauge.GetValue() < self.gauge.GetRange():
+				self.Update(self.gauge.GetValue() + 1)
+			else:
+				self.stop_timer()
+				self.Update(self.gauge.GetRange(),
+							"Finished. You may now close this window.")
+		else:
 			self.Pulse("Aborting...")
 			if not hasattr(self, "delayed_stop"):
 				self.delayed_stop = wx.CallLater(3000, self.stop_timer)
 				wx.CallLater(3000, self.Pulse, 
 							 "Aborted. You may now close this window.")
 	
+	def Pulse(self, msg=None):
+		if msg and msg != self.msg.Label:
+			self.msg.SetLabel(msg)
+			self.msg.Refresh()
+			self.msg.Update()
+		self.gauge.Pulse()
+		return self.keepGoing, self.skip
+	
+	def Resume(self):
+		self.keepGoing = True
+		if hasattr(self, "cancel"):
+			self.cancel.Enable()
+		self.pause_continue.Enable()
+	
+	def Update(self, value, msg=None):
+		if msg and msg != self.msg.Label:
+			self.msg.SetLabel(msg)
+			self.msg.Refresh()
+			self.msg.Update()
+		self.gauge.SetValue(value)
+		return self.keepGoing, self.skip
+	
+	UpdatePulse = Pulse
+	
+	def elapsed_time_handler(self, event):
+		self.elapsed_time.Label = strftime("%H:%M:%S",
+										   gmtime(time() - self.time))
+	
 	def key_handler(self, event):
 		pass
-	
+
+	def pause_continue_handler(self, event=None):
+		self.paused = not self.paused
+		if self.paused:
+			self.pause_continue.Label = lang.getstr("continue")
+		else:
+			self.pause_continue.Label = lang.getstr("pause")
+		self.pause_continue.Enable(not event)
+		self.Layout()
+
 	def start_timer(self, ms=50):
 		self.timer.Start(ms)
+		if hasattr(self, "elapsed_timer"):
+			self.elapsed_timer.Start(1000)
 	
 	def stop_timer(self):
 		self.timer.Stop()
+		if hasattr(self, "elapsed_timer"):
+			self.elapsed_timer.Stop()
 
 
 class SimpleBook(labelbook.FlatBookBase):
